@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -19,16 +19,16 @@
  ** along with PhotoQt. If not, see <http://www.gnu.org/licenses/>.      **
  **                                                                      **
  **************************************************************************/
-
-#ifndef PQCFILEFOLDERMODEL_H
-#define PQCFILEFOLDERMODEL_H
+#pragma once
 
 #include <QObject>
 #include <pqc_filefoldermodelcache.h>
 #include <QTimer>
 #include <QMimeDatabase>
 #include <QSize>
-#include <QtQmlIntegration>
+#include <QQmlEngine>
+
+#include <pqc_filefoldermodelCPP.h>
 
 class QSize;
 class QFileSystemWatcher;
@@ -36,14 +36,11 @@ class QFileSystemWatcher;
 class PQCFileFolderModel : public QObject {
 
     Q_OBJECT
+    QML_ELEMENT
     QML_SINGLETON
 
 public:
-    static PQCFileFolderModel& get();
-
-    PQCFileFolderModel(PQCFileFolderModel const&)     = delete;
-    void operator=(PQCFileFolderModel const&) = delete;
-
+    PQCFileFolderModel(QObject *parent = 0);
     ~PQCFileFolderModel();
 
     /********************************************/
@@ -57,6 +54,10 @@ public:
     QString getFolderFileDialog();
     void setFolderFileDialog(QString val);
 
+    Q_PROPERTY(bool firstFolderMainViewLoaded READ getFirstFolderMainViewLoaded WRITE setFirstFolderMainViewLoaded NOTIFY firstFolderMainViewLoadedChanged)
+    bool getFirstFolderMainViewLoaded();
+    void setFirstFolderMainViewLoaded(bool val);
+
     /********************************************/
     /********************************************/
 
@@ -96,9 +97,21 @@ public:
     bool getReadArchiveOnly();
     void setReadArchiveOnly(bool c);
 
-    Q_PROPERTY(QStringList extraFoldersToLoad READ getExtraFoldersToLoad WRITE setExtraFoldersToLoad NOTIFY extraFoldersToLoadChanged)
-    QStringList getExtraFoldersToLoad();
-    void setExtraFoldersToLoad(QStringList val);
+    Q_PROPERTY(QStringList virtualFolders READ getVirtualFolders WRITE setVirtualFolders NOTIFY virtualFoldersChanged)
+    QStringList getVirtualFolders();
+    void setVirtualFolders(QStringList val);
+
+    Q_PROPERTY(QStringList virtualFiles READ getVirtualFiles WRITE setVirtualFiles NOTIFY virtualFilesChanged)
+    QStringList getVirtualFiles();
+    void setVirtualFiles(QStringList val);
+
+    Q_PROPERTY(bool loadVirtualFolderFileDialog READ getLoadVirtualFolderFileDialog WRITE setLoadVirtualFolderFileDialog NOTIFY loadVirtualFolderFileDialogChanged)
+    bool getLoadVirtualFolderFileDialog();
+    void setLoadVirtualFolderFileDialog(bool c);
+
+    Q_PROPERTY(bool loadVirtualFolderMainView READ getLoadVirtualFolderMainView WRITE setLoadVirtualFolderMainView NOTIFY loadVirtualFolderMainViewChanged)
+    bool getLoadVirtualFolderMainView();
+    void setLoadVirtualFolderMainView(bool c);
 
     /********************************************/
     /********************************************/
@@ -142,6 +155,7 @@ public:
     Q_PROPERTY(int currentIndex READ getCurrentIndex WRITE setCurrentIndex NOTIFY currentIndexChanged)
     int getCurrentIndex();
     void setCurrentIndex(int val);
+    Q_INVOKABLE int getIndexOf(QString file);
 
     Q_PROPERTY(int currentIndexNoDelay READ getCurrentIndexNoDelay NOTIFY currentIndexNoDelayChanged)
     int getCurrentIndexNoDelay();
@@ -170,10 +184,13 @@ public:
     Q_PROPERTY(QString arcFile READ getArcFile NOTIFY arcFileChanged)
     QString getArcFile();
 
+    Q_PROPERTY(bool justLeftViewerMode MEMBER m_justLeftViewerMode NOTIFY justLeftViewerModeChanged)
+    Q_PROPERTY(bool activeViewerMode MEMBER m_activeViewerMode NOTIFY activeViewerModeChanged)
+
     /********************************************/
     /********************************************/
 
-    Q_INVOKABLE void advancedSortMainView();
+    Q_INVOKABLE void advancedSortMainView(QString advSortCriteria, bool advSortAscending, QString advSortQuality, QStringList advDateCriteria);
     Q_INVOKABLE void advancedSortMainViewCANCEL();
     Q_INVOKABLE void forceReloadMainView();
     Q_INVOKABLE void forceReloadFileDialog();
@@ -182,8 +199,9 @@ public:
     Q_INVOKABLE void removeAllUserFilter();
     Q_INVOKABLE bool isUserFilterSet();
     Q_INVOKABLE void enableViewerMode(int page = 0);
-    Q_INVOKABLE void disableViewerMode();
+    Q_INVOKABLE void disableViewerMode(bool bufferDisabling = true);
     Q_INVOKABLE QString getFirstMatchFileDialog(QString partial);
+    Q_INVOKABLE void loadNextMatchOfSearch(const QString search);
 
     /********************************************/
 
@@ -191,18 +209,16 @@ public:
 
     /********************************************/
 
-    QTimer *timerNotifyCurrentIndexChanged;
-
 private:
-    PQCFileFolderModel(QObject *parent = 0);
-
     PQCFileFolderModelCache cache;
 
     QFileSystemWatcher *watcherMainView;
     QFileSystemWatcher *watcherFileDialog;
 
+
     QString m_fileInFolderMainView;
     QString m_folderFileDialog;
+    bool m_firstFolderMainViewLoaded;
     int m_countMainView;
     int m_countFoldersFileDialog;
     int m_countFilesFileDialog;
@@ -211,7 +227,10 @@ private:
     bool m_readDocumentOnly;
     bool m_readArchiveOnly;
     bool m_includeFilesInSubFolders;
-    QStringList m_extraFoldersToLoad;
+    QStringList m_virtualFolders;
+    QStringList m_virtualFiles;
+    bool m_loadVirtualFolderFileDialog;
+    bool m_loadVirtualFolderMainView;
 
     QStringList m_entriesMainView;
     QStringList m_entriesFileDialog;
@@ -223,6 +242,8 @@ private:
     QSize m_imageResolutionFilter;
     qint64 m_fileSizeFilter;
     bool m_filterCurrentlyActive;
+    bool m_justLeftViewerMode;
+    bool m_activeViewerMode;
 
     int m_currentIndex;
     int m_currentIndexNoDelay;
@@ -239,6 +260,9 @@ private:
     QTimer *loadDelayMainView;
     QTimer *loadDelayFileDialog;
 
+    QTimer *timerNotifyCurrentIndexChanged;
+    QTimer *timerResetJustLeftViewerMode;
+
     QStringList getAllFolders(QString folder, bool forceShowHidden = false);
     QStringList getAllFiles(QString folder, bool ignoreFiltersExceptDefault = false, bool enforceOnlyIncludingThisFolder = false);
 
@@ -264,8 +288,6 @@ private Q_SLOTS:
     void loadDataFileDialog();
     void handleNewDataLoadedMainView();
 
-    void handleSettingsChanges(const QString &key, const QVariant &value);
-
 Q_SIGNALS:
     void newDataLoadedMainView();
     void newDataLoadedFileDialog();
@@ -279,6 +301,7 @@ Q_SIGNALS:
     void entriesMainViewChanged();
     void entriesFileDialogChanged();
     void fileInFolderMainViewChanged();
+    void firstFolderMainViewLoadedChanged();
     void folderFileDialogChanged();
     void nameFiltersChanged();
     void restrictToSuffixesChanged();
@@ -292,7 +315,10 @@ Q_SIGNALS:
     void readDocumentOnlyChanged();
     void readArchiveOnlyChanged();
     void includeFilesInSubFoldersChanged();
-    void extraFoldersToLoadChanged();
+    void virtualFoldersChanged();
+    void virtualFilesChanged();
+    void loadVirtualFolderFileDialogChanged();
+    void loadVirtualFolderMainViewChanged();
     void advancedSortDoneChanged();
     void currentIndexChanged();
     void currentIndexNoDelayChanged();
@@ -304,7 +330,7 @@ Q_SIGNALS:
     void pdfNumChanged();
     void arcNameChanged();
     void arcFileChanged();
+    void justLeftViewerModeChanged();
+    void activeViewerModeChanged();
 
 };
-
-#endif
