@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -19,9 +19,7 @@
  ** along with PhotoQt. If not, see <http://www.gnu.org/licenses/>.      **
  **                                                                      **
  **************************************************************************/
-
-#ifndef PQCLOADIMAGEDEVIL_H
-#define PQCLOADIMAGEDEVIL_H
+#pragma once
 
 #include <QMutex>
 
@@ -42,7 +40,7 @@ public:
         void operator=(PQCLoadImageDevilMutex const&) = delete;
 #ifdef PQMDEVIL
         // DevIL is not threadsafe -> this ensures only one image is loaded at a time
-        QMutex devilMutex;
+        mutable QMutex devilMutex;
 #endif
 private:
         PQCLoadImageDevilMutex() {}
@@ -63,5 +61,3 @@ private:
 #endif
 
 };
-
-#endif // PQCLOADIMAGEDEVIL_H
