@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -19,9 +19,9 @@
  ** along with PhotoQt. If not, see <http://www.gnu.org/licenses/>.      **
  **                                                                      **
  **************************************************************************/
+#pragma once
 
-#ifndef PQC_PHOTOSPHERERENDERER
-#define PQC_PHOTOSPHERERENDERER
+#ifdef PQMPHOTOSPHERE
 
 #include <QQuickFramebufferObject>
 #include <QMatrix4x4>
