@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -19,9 +19,7 @@
  ** along with PhotoQt. If not, see <http://www.gnu.org/licenses/>.      **
  **                                                                      **
  **************************************************************************/
-
-#ifndef PQCVALIDATE_H
-#define PQCVALIDATE_H
+#pragma once
 
 #include <QObject>
 
@@ -37,12 +35,9 @@ public:
     bool validateContextMenuDatabase();
     bool validateImageFormatsDatabase();
     bool validateSettingsDatabase();
-    bool validateShortcutsDatabase();
     bool validateSettingsValues();
-    bool validateDirectories(QString thumb_cache_basedir);
+    bool validateShortcutsDatabase();
+    bool validateDirectories(const QString &thumb_cache_basedir);
     bool validateLocationDatabase();
-    bool validateImgurHistoryDatabase();
 
 };
-
-#endif // PQVALIDATE_H
