@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -19,12 +19,9 @@
  ** along with PhotoQt. If not, see <http://www.gnu.org/licenses/>.      **
  **                                                                      **
  **************************************************************************/
-
-#ifndef PQCSCRIPTSCHROMECAST_H
-#define PQCSCRIPTSCHROMECAST_H
+#pragma once
 
 #include <QObject>
-#include <QtQmlIntegration>
 
 class PQCLocalHttpServer;
 class QProcess;
@@ -32,41 +29,29 @@ class QProcess;
 class PQCScriptsChromeCast : public QObject {
 
     Q_OBJECT
-    QML_SINGLETON
 
 public:
     static PQCScriptsChromeCast& get() {
         static PQCScriptsChromeCast instance;
         return instance;
     }
-    ~PQCScriptsChromeCast();
 
-    PQCScriptsChromeCast(PQCScriptsChromeCast const&)     = delete;
+    PQCScriptsChromeCast(PQCScriptsChromeCast const&) = delete;
     void operator=(PQCScriptsChromeCast const&) = delete;
 
-    Q_PROPERTY(QVariantList availableDevices READ getAvailableDevices WRITE setAvailableDevices NOTIFY availableDevicesChanged)
-    void setAvailableDevices(QVariantList val);
-    QVariantList getAvailableDevices();
+    QVariantList getAvailableDevices() { return m_availableDevices; }
+    QString getCurDeviceName() { return m_curDeviceName; }
+    bool getInDiscovery() { return m_inDiscovery; }
+    bool getConnected() { return m_connected; }
 
-    Q_PROPERTY(QString curDeviceName READ getCurDeviceName WRITE setCurDeviceName NOTIFY curDeviceNameChanged)
-    void setCurDeviceName(QString val);
-    QString getCurDeviceName();
-
-    Q_PROPERTY(bool inDiscovery READ getInDiscovery WRITE setInDiscovery NOTIFY inDiscoveryChanged)
-    int getInDiscovery();
-    void setInDiscovery(bool val);
-
-    Q_PROPERTY(bool connected READ getConnected WRITE setConnected NOTIFY connectedChanged)
-    int getConnected();
-    void setConnected(bool val);
-
-    Q_INVOKABLE bool startDiscovery();
-    Q_INVOKABLE bool connectToDevice(int deviceId);
-    Q_INVOKABLE bool castImage(QString filename);
-    Q_INVOKABLE bool disconnect();
+    bool startDiscovery();
+    bool connectToDevice(int deviceId);
+    bool castImage(QString filename);
+    bool disconnect();
 
 private:
-    PQCScriptsChromeCast();
+    explicit PQCScriptsChromeCast();
+    ~PQCScriptsChromeCast();
 
     QProcess *procDiscovery;
     QProcess *procCast;
@@ -88,11 +73,8 @@ private Q_SLOTS:
 
 Q_SIGNALS:
     void availableDevicesChanged();
-    void selectedDeviceChanged();
     void inDiscoveryChanged();
     void connectedChanged();
     void curDeviceNameChanged();
 
 };
-
-#endif
