@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -19,36 +19,34 @@
  ** along with PhotoQt. If not, see <http://www.gnu.org/licenses/>.      **
  **                                                                      **
  **************************************************************************/
-
-#ifndef PQCSCRIPTSCONTEXTMENU_H
-#define PQCSCRIPTSCONTEXTMENU_H
+#pragma once
 
 #include <QSqlDatabase>
 #include <QObject>
-#include <QtQmlIntegration>
 
 class PQCScriptsContextMenu : public QObject {
 
     Q_OBJECT
-    QML_SINGLETON
 
 public:
     static PQCScriptsContextMenu& get() {
         static PQCScriptsContextMenu instance;
         return instance;
     }
-    ~PQCScriptsContextMenu();
 
-    PQCScriptsContextMenu(PQCScriptsContextMenu const&)     = delete;
+    PQCScriptsContextMenu(PQCScriptsContextMenu const&) = delete;
     void operator=(PQCScriptsContextMenu const&) = delete;
 
-    Q_INVOKABLE QVariantList getEntries();
-    Q_INVOKABLE void setEntries(QVariantList entries);
+    QVariantList getEntries();
+    void setEntries(QVariantList entries);
 
-    Q_INVOKABLE QVariantList detectSystemEntries();
+    QVariantList detectSystemEntries();
+
+    void closeDatabase();
 
 private:
     PQCScriptsContextMenu();
+    ~PQCScriptsContextMenu();
 
     QSqlDatabase db;
 
@@ -56,5 +54,3 @@ Q_SIGNALS:
     void customEntriesChanged();
 
 };
-
-#endif
