@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -19,47 +19,55 @@
  ** along with PhotoQt. If not, see <http://www.gnu.org/licenses/>.      **
  **                                                                      **
  **************************************************************************/
-
-#ifndef PQCSCRIPTSFILEDIALOG_H
-#define PQCSCRIPTSFILEDIALOG_H
+#pragma once
 
 #include <QObject>
 #include <QHash>
-#include <QtQmlIntegration>
+#include <QMutex>
 
 class QJSValue;
 
 class PQCScriptsFileDialog : public QObject {
 
     Q_OBJECT
-    QML_SINGLETON
 
 public:
     static PQCScriptsFileDialog& get() {
         static PQCScriptsFileDialog instance;
         return instance;
     }
-    ~PQCScriptsFileDialog();
 
     PQCScriptsFileDialog(PQCScriptsFileDialog const&)     = delete;
     void operator=(PQCScriptsFileDialog const&) = delete;
 
-    Q_INVOKABLE QVariantList getDevices();
-    Q_INVOKABLE QVariantList getPlaces(bool performEmptyCheck = true);
+    // get data
+    QVariantList getDevices();
+    QVariantList getPlaces(bool performEmptyCheck = true);
     QString getUniquePlacesId();
-    Q_INVOKABLE bool setLastLocation(QString path);
-    Q_INVOKABLE QString getLastLocation();
-    unsigned int _getNumberOfFilesInFolder(QString path);
-    Q_INVOKABLE void getNumberOfFilesInFolder(QString path, const QJSValue &callback);
-    Q_INVOKABLE void movePlacesEntry(QString id, bool moveDown, int howmany);
-    Q_INVOKABLE void addPlacesEntry(QString path, int pos, QString titlestring = "", QString icon = "folder", bool isSystemItem = false);
-    Q_INVOKABLE void hidePlacesEntry(QString id, bool hidden);
-    Q_INVOKABLE void deletePlacesEntry(QString id);
+
+    // last location
+    // this value is set in PQCFileFolderModel::setFolderFileDialog() when the custom filedialog is used
+    QString getLastLocation();
+    // The following one ONLY needs to be called from the integrated ui IF the native filedialog is used!
+    void setLastLocation(QString fname);
+
+    // count folder files
+    unsigned int getNumberOfFilesInFolder(const QString &path);
+
+    // places methods
+    void movePlacesEntry(QString id, bool moveDown, int howmany);
+    void addPlacesEntry(QString path, int pos, QString titlestring = "", QString icon = "folder", bool isSystemItem = false);
+    void hidePlacesEntry(QString id, bool hidden);
+    void deletePlacesEntry(QString id);
+
+    // quickly move between folders
+    QString getSiblingFolder(QString currentFolder, const int direction);
 
 private:
     PQCScriptsFileDialog();
+    ~PQCScriptsFileDialog();
+
+    mutable QMutex cacheMutex;
     QHash<QString,int> cacheNumberOfFilesInFolder;
 
 };
-
-#endif
