@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2024 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -19,27 +19,31 @@
  ** along with PhotoQt. If not, see <http://www.gnu.org/licenses/>.      **
  **                                                                      **
  **************************************************************************/
-
-#ifndef PQCSCRIPTSMETADATA_H
-#define PQCSCRIPTSMETADATA_H
+#pragma once
 
 #include <QObject>
 
+/*************************************************************/
+/*************************************************************/
+//
+// this class is used in both C++ and QML code
+// thus there is a WRAPPER for QML available
+//
+/*************************************************************/
+/*************************************************************/
+
 class PQCScriptsMetaData : public QObject {
 
     Q_OBJECT
 
 public:
-    static PQCScriptsMetaData& get() {
-        static PQCScriptsMetaData instance;
-        return instance;
-    }
-    ~PQCScriptsMetaData();
+    static PQCScriptsMetaData& get();
+    virtual ~PQCScriptsMetaData();
 
     PQCScriptsMetaData(PQCScriptsMetaData const&)     = delete;
     void operator=(PQCScriptsMetaData const&) = delete;
 
-    QString analyzeDateTimeOriginal(const QString val);
+    QString analyzeDateTimeOriginal(const QString dateTime, const QString offset);
     QString analyzeExposureTime(const QString val);
     QString analyzeFlash(const QString val);
     QString analyzeSceneCaptureType(const QString val);
@@ -48,19 +52,21 @@ public:
     QString analyzeLightSource(const QString val);
     QString analyzeGPS(QString latRef, QString lat, QString lonRef, QString lon);
     QPointF convertGPSToDecimal(QString gpsLatRef, QString gpsLat, QString gpsLonRef, QString gpsLon);
+    QPointF convertGPSToPoint(QString gps);
 
-    Q_INVOKABLE int getExifOrientation(QString path);
+    /***************************************************************/
+    // only the following will be called from QML (through wrapper)
 
-    Q_INVOKABLE QString convertGPSToDecimalForOpenStreetMap(QString gps);
-    Q_INVOKABLE QPointF convertGPSToPoint(QString gps);
+    int getExifOrientation(QString path);
 
-    Q_INVOKABLE bool areFaceTagsSupported(QString filename);
-    Q_INVOKABLE QVariantList getFaceTags(QString filename);
-    Q_INVOKABLE void setFaceTags(QString filename, QVariantList tags);
+    QString convertGPSToDecimalForOpenStreetMap(QString gps);
+    QString convertGPSDecimalToDegree(double lat, double lon);
+
+    bool areFaceTagsSupported(QString filename);
+    QVariantList getFaceTags(QString filename);
+    void setFaceTags(QString filename, QVariantList tags);
 
 private:
     PQCScriptsMetaData();
 
 };
-
-#endif
