@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -19,40 +19,76 @@
  ** along with PhotoQt. If not, see <http://www.gnu.org/licenses/>.      **
  **                                                                      **
  **************************************************************************/
-
-#ifndef PQCSCRIPTSSHORTCUTS_H
-#define PQCSCRIPTSSHORTCUTS_H
+#pragma once
 
 #include <QObject>
-#include <QMap>
-#include <QtQmlIntegration>
+#include <QHash>
 
 class PQCScriptsShortcuts : public QObject {
 
     Q_OBJECT
-    QML_SINGLETON
 
 public:
     static PQCScriptsShortcuts& get() {
         static PQCScriptsShortcuts instance;
         return instance;
     }
-    ~PQCScriptsShortcuts();
 
     PQCScriptsShortcuts(PQCScriptsShortcuts const&)     = delete;
     void operator=(PQCScriptsShortcuts const&) = delete;
 
-    Q_INVOKABLE void executeExternal(QString exe, QString args, QString currentfile);
+    void executeExternal(QString exe, QString args, QString currentfile);
 
-    Q_INVOKABLE QStringList analyzeModifier(Qt::KeyboardModifiers mods);
-    Q_INVOKABLE QString analyzeMouseWheel(QPoint angleDelta);
-    Q_INVOKABLE QString analyzeMouseButton(Qt::MouseButton button);
-    Q_INVOKABLE QString analyzeMouseDirection(QPoint prevPoint, QPoint curPoint);
-    Q_INVOKABLE QString analyzeKeyPress(Qt::Key key);
+    QStringList analyzeModifier(Qt::KeyboardModifiers mods);
+    QString analyzeMouseWheel(QPoint angleDelta);
+    QString analyzeMouseButton(Qt::MouseButton button);
+    QString analyzeMouseDirection(QPoint prevPoint, QPoint curPoint);
+    QString analyzeKeyPress(Qt::Key key);
+
+    void setCurrentTimestamp();
+    int getCurrentTimestampDiffLessThan(int threshold);
+
+    QString translateShortcut(QString combo);
+    QString translateMouseDirection(QStringList combo);
 
 private:
     PQCScriptsShortcuts();
+    ~PQCScriptsShortcuts();
+
+    qint64 m_lastInternalShortcutExecuted;
+
+    QHash<QString,QString> m_keyStrings;
+    QHash<QString,QString> m_mouseStrings;
+
+Q_SIGNALS:
+
+    void sendShortcutShowGlobalContextMenuAt(QPoint pos);
+    void sendShortcutDismissGlobalContextMenu();
+
+    void executeInternalCommand(QString cmd);
+    void executeInternalCommandWithMousePos(QString cmd, QPoint pos);
+
+    void sendShortcutShowNextImage();
+    void sendShortcutShowPrevImage();
+    void sendShortcutShowNextArcDocImage();
+    void sendShortcutShowPrevArcDocImage();
+    void sendShortcutShowFirstImage();
+    void sendShortcutShowLastImage();
+    void sendShortcutShowRandomImage();
+    void sendShortcutShowFile(QString path);
+
+    void sendShortcutZoomIn(QPoint mousePos, QPoint wheelDelta);
+    void sendShortcutZoomOut(QPoint mousePos, QPoint wheelDelta);
+    void sendShortcutZoomReset();
+    void sendShortcutZoomActual();
+    void sendShortcutZoomKenBurns();
+
+    void sendShortcutRotateClock();
+    void sendShortcutRotateAntiClock();
+    void sendShortcutRotateReset();
+
+    void sendShortcutMirrorHorizontal();
+    void sendShortcutMirrorVertical();
+    void sendShortcutMirrorReset();
 
 };
-
-#endif
