@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -20,22 +20,22 @@
  **                                                                      **
  **************************************************************************/
 
-#include <thread>
-
 #include <pqc_loadimage.h>
 #include <pqc_loadimage_archive.h>
 #include <pqc_imagecache.h>
 #include <scripts/pqc_scriptsimages.h>
-#include <pqc_settings.h>
+#include <scripts/pqc_scriptscolorprofiles.h>
+#include <pqc_settingscpp.h>
 #include <pqc_configfiles.h>
 #include <pqc_loadimage.h>
 #include <pqc_imageformats.h>
-#include <pqc_notify.h>
+#include <pqc_notify_cpp.h>
 
 #include <QSize>
 #include <QtDebug>
 #include <QFileInfo>
 #include <QImage>
+#include <QProcess>
 
 #ifdef PQMLIBARCHIVE
 #include <archive.h>
@@ -56,7 +56,7 @@ QSize PQCLoadImageArchive::loadSize(QString filename) {
         archivefile = parts.at(1);
         compressedFilename = parts.at(0);
     } else {
-        QStringList cont = PQCScriptsImages::get().listArchiveContent(archivefile);
+        QStringList cont = PQCScriptsImages::get().listArchiveContentWithoutThread(archivefile);
         if(cont.length() == 0) {
             qWarning() << "Unable to list contents of archive file...";
             return QSize();
@@ -132,7 +132,10 @@ QSize PQCLoadImageArchive::loadSize(QString filename) {
                 dir.mkpath(info.absolutePath());
 
             // write buffer to file
-            file.open(QIODevice::WriteOnly);
+            if(!file.open(QIODevice::WriteOnly)) {
+                qWarning() << "Unable to load archive file to buffer.";
+                return QSize();
+            }
             QDataStream out(&file);   // we will serialize the data into the file
             out.writeRawData((const char*) buff,size);
             file.close();
@@ -174,7 +177,7 @@ QString PQCLoadImageArchive::load(QString filename, QSize maxSize, QSize &origSi
         archivefile = parts.at(1);
         compressedFilename = parts.at(0);
     } else {
-        QStringList cont = PQCScriptsImages::get().listArchiveContent(archivefile);
+        QStringList cont = PQCScriptsImages::get().listArchiveContentWithoutThread(archivefile);
         if(cont.length() == 0) {
             errormsg = "Unable to list contents of archive file...";
             qWarning() << errormsg;
@@ -193,7 +196,7 @@ QString PQCLoadImageArchive::load(QString filename, QSize maxSize, QSize &origSi
     const QString suffix = info.suffix().toLower();
 
 #ifndef Q_OS_WIN
-    if(PQCSettings::get()["filetypesExternalUnrar"].toBool() && (suffix == "cbr" || suffix == "rar")) {
+    if(PQCSettingsCPP::get().getFiletypesExternalUnrar() && (suffix == "cbr" || suffix == "rar")) {
 
         QProcess which;
         which.setStandardOutputFile(QProcess::nullDevice());
@@ -219,8 +222,8 @@ QString PQCLoadImageArchive::load(QString filename, QSize maxSize, QSize &origSi
 
                 // cache image before potentially scaling it
                 if(!img.isNull()) {
-                    PQCScriptsImages::get().applyColorProfile(filename, img);
-                    PQCImageCache::get().saveImageToCache(filename, PQCScriptsImages::get().getColorProfileFor(filename), &img);
+                    PQCScriptsColorProfiles::get().applyColorProfile(filename, img);
+                    PQCImageCache::get().saveImageToCache(filename, PQCScriptsColorProfiles::get().getColorProfileFor(filename), &img);
                 }
 
                 // Scale image if necessary
@@ -311,7 +314,11 @@ QString PQCLoadImageArchive::load(QString filename, QSize maxSize, QSize &origSi
                 dir.mkpath(info.absolutePath());
 
             // write buffer to file
-            file.open(QIODevice::WriteOnly);
+            if(!file.open(QIODevice::WriteOnly)) {
+                const QString err = "Unable to load archive file to buffer.";
+                qWarning() << err;
+                return err;
+            }
             QDataStream out(&file);   // we will serialize the data into the file
             out.writeRawData((const char*) buff,size);
             file.close();
@@ -341,8 +348,14 @@ QString PQCLoadImageArchive::load(QString filename, QSize maxSize, QSize &origSi
 
     // cache image before potentially scaling it
     if(!img.isNull()) {
-        PQCScriptsImages::get().applyColorProfile(filename, img);
-        PQCImageCache::get().saveImageToCache(filename, PQCScriptsImages::get().getColorProfileFor(filename), &img);
+
+        if(PQCSettingsCPP::get().getMetadataAutoRotation()) {
+            // apply transformations if any
+            PQCScriptsImages::get().applyExifOrientation(filename, img);
+        }
+
+        PQCScriptsColorProfiles::get().applyColorProfile(filename, img);
+        PQCImageCache::get().saveImageToCache(filename, PQCScriptsColorProfiles::get().getColorProfileFor(filename), &img);
     }
 
     // Scale image if necessary
