@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2024 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -23,13 +23,14 @@
 #include <pqc_loadimage_freeimage.h>
 #include <pqc_imagecache.h>
 #include <scripts/pqc_scriptsimages.h>
-#include <pqc_notify.h>
+#include <scripts/pqc_scriptscolorprofiles.h>
+#include <pqc_notify_cpp.h>
+#include <pqc_settingscpp.h>
 
 #include <QString>
 #include <QImage>
 #include <QSize>
 #include <QtDebug>
-#include <QCoreApplication>
 
 #ifdef PQMFREEIMAGE
 #include <FreeImage.h>
@@ -126,9 +127,13 @@ QString PQCLoadImageFreeImage::load(QString filename, QSize maxSize, QSize &orig
         return errormsg;
     }
 
-    if(!PQCScriptsImages::get().applyColorProfile(filename, img))
-        Q_EMIT PQCNotify::get().showNotificationMessage(QCoreApplication::translate("imageprovider", "The selected color profile could not be applied."));
-    PQCImageCache::get().saveImageToCache(filename, PQCScriptsImages::get().getColorProfileFor(filename), &img);
+    if(PQCSettingsCPP::get().getMetadataAutoRotation()) {
+        // apply transformations if any
+        PQCScriptsImages::get().applyExifOrientation(filename, img);
+    }
+
+    PQCScriptsColorProfiles::get().applyColorProfile(filename, img);
+    PQCImageCache::get().saveImageToCache(filename, PQCScriptsColorProfiles::get().getColorProfileFor(filename), &img);
 
     // Scale image if necessary
     if(maxSize.width() != -1) {
