@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2024 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -32,11 +32,14 @@
 #include <pqc_loadimage_devil.h>
 #include <pqc_loadimage_freeimage.h>
 #include <pqc_loadimage_video.h>
+#include <pqc_loadimage_libsai.h>
 
 #include <pqc_loadimage.h>
 #include <pqc_settings.h>
 #include <pqc_imageformats.h>
 #include <scripts/pqc_scriptsimages.h>
+#include <scripts/pqc_scriptscolorprofiles.h>
+
 #include <QSize>
 #include <QImage>
 #include <QFileInfo>
@@ -58,72 +61,79 @@ QSize PQCLoadImage::load(QString filename) {
 
     // check image cache, we might be done right here
     QImage img;
-    if(PQCImageCache::get().getCachedImage(filename, PQCScriptsImages::get().getColorProfileFor(filename), img))
+    if(PQCImageCache::get().getCachedImage(filename, PQCScriptsColorProfiles::get().getColorProfileFor(filename), img))
         return img.size();
 
     // for easier access below
-    QString suffix = info.suffix().toLower();
+    QString suffix1 = info.suffix().toLower();
+    QString suffix2 = info.completeSuffix().toLower();
 
     //////////////////////////////////////////////
     //////////////////////////////////////////////
     // first we check for filename suffix matches
 
-    // we need to explicitely set the initial size to be 0,0 otherwise the isNull check below will always fail
+    // we need to explicitly set the initial size to be 0,0 otherwise the isNull check below will always fail
     QSize sze(0,0);
 
     // resvg trumps Qt's SVG engine
 #ifdef PQMRESVG
-    if(PQCImageFormats::get().getEnabledFormatsResvg().contains(suffix))
+    if(PQCImageFormats::get().getEnabledFormatsResvg().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsResvg().contains(suffix2))
         sze = PQCLoadImageResvg::loadSize(filename);
 #endif
 
 #ifdef PQMPOPPLER
-    if(sze.isNull() && PQCImageFormats::get().getEnabledFormatsPoppler().contains(suffix))
+    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsPoppler().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsPoppler().contains(suffix2)))
         sze = PQCLoadImagePoppler::loadSize(filename);
 #endif
 
 #ifdef PQMQTPDF
-    if(sze.isNull() && PQCImageFormats::get().getEnabledFormatsPoppler().contains(suffix))
+    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsPoppler().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsPoppler().contains(suffix2)))
         sze = PQCLoadImageQtPDF::loadSize(filename);
 #endif
 
-    if(sze.isNull() && PQCImageFormats::get().getEnabledFormatsQt().contains(suffix))
+    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsQt().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsQt().contains(suffix2)))
         sze = PQCLoadImageQt::loadSize(filename);
 
 #ifdef PQMRAW
-    if(sze.isNull() && PQCImageFormats::get().getEnabledFormatsLibRaw().contains(suffix))
+    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsLibRaw().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsLibRaw().contains(suffix2)))
         sze = PQCLoadImageRAW::loadSize(filename);
 #endif
 
 #ifdef PQMLIBARCHIVE
-    if(sze.isNull() && PQCImageFormats::get().getEnabledFormatsLibArchive().contains(suffix))
+    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsLibArchive().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsLibArchive().contains(suffix2)))
         sze = PQCLoadImageArchive::loadSize(filename);
 #endif
 
-    if(sze.isNull() && PQCImageFormats::get().getEnabledFormatsXCFTools().contains(suffix))
+#ifdef PQMLIBSAI
+    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsLibsai().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsLibsai().contains(suffix2)))
+        sze = PQCLoadImageLibsai::loadSize(filename);
+#endif
+
+    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsXCFTools().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsXCFTools().contains(suffix2)))
         sze = PQCLoadImageXCF::loadSize(filename);
 
 #if defined(PQMIMAGEMAGICK) || defined(PQMGRAPHICSMAGICK)
-    if(sze.isNull() && PQCImageFormats::get().getEnabledFormatsMagick().contains(suffix))
+    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsMagick().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsMagick().contains(suffix2)))
         sze = PQCLoadImageMagick::loadSize(filename);
 #endif
 
 #ifdef PQMLIBVIPS
-    if(sze.isNull() && PQCImageFormats::get().getEnabledFormatsLibVips().contains(suffix))
+    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsLibVips().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsLibVips().contains(suffix2)))
         sze = PQCLoadImageLibVips::loadSize(filename);
 #endif
 
 #ifdef PQMFREEIMAGE
-    if(sze.isNull() && PQCImageFormats::get().getEnabledFormatsFreeImage().contains(suffix))
+    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsFreeImage().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsFreeImage().contains(suffix2)))
         sze = PQCLoadImageFreeImage::loadSize(filename);
 #endif
 
 #ifdef PQMDEVIL
-    if(sze.isNull() && PQCImageFormats::get().getEnabledFormatsDevIL().contains(suffix))
+    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsDevIL().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsDevIL().contains(suffix2)))
         sze = PQCLoadImageDevil::loadSize(filename);
 #endif
 
-    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsVideo().contains(suffix) || PQCImageFormats::get().getEnabledFormatsLibmpv().contains(suffix)))
+    if(sze.isNull() && (PQCImageFormats::get().getEnabledFormatsVideo().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsLibmpv().contains(suffix1) ||
+                        PQCImageFormats::get().getEnabledFormatsVideo().contains(suffix2) || PQCImageFormats::get().getEnabledFormatsLibmpv().contains(suffix2)))
         sze = PQCLoadImageVideo::loadSize(filename);
 
 
@@ -166,6 +176,11 @@ QSize PQCLoadImage::load(QString filename) {
                 sze = PQCLoadImageArchive::loadSize(filename);
 #endif
 
+#ifdef PQMLIBSAI
+            if(sze.isNull() && PQCImageFormats::get().getEnabledMimeTypesLibsai().contains(mimetype))
+                sze = PQCLoadImageLibsai::loadSize(filename);
+#endif
+
             if(sze.isNull() && PQCImageFormats::get().getEnabledMimeTypesXCFTools().contains(mimetype))
                 sze = PQCLoadImageXCF::loadSize(filename);
 
@@ -227,7 +242,7 @@ QString PQCLoadImage::load(QString filename, QSize requestedSize, QSize &origSiz
         filename = info.symLinkTarget();
 
     // check image cache, we might be done right here
-    if(PQCImageCache::get().getCachedImage(filename, PQCScriptsImages::get().getColorProfileFor(filename), img)) {
+    if(PQCImageCache::get().getCachedImage(filename, PQCScriptsColorProfiles::get().getColorProfileFor(filename), img)) {
         origSize = img.size();
         if(requestedSize.width() > 2 && requestedSize.height() > 2 && origSize.width() > requestedSize.width() && origSize.height() > requestedSize.height())
             img = img.scaled(requestedSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
@@ -237,7 +252,8 @@ QString PQCLoadImage::load(QString filename, QSize requestedSize, QSize &origSiz
     QString err = "";
 
     // for easier access below
-    QString suffix = info.suffix().toLower();
+    QString suffix1 = info.suffix().toLower();
+    QString suffix2 = info.completeSuffix().toLower();
 
     //////////////////////////////////////////////
     //////////////////////////////////////////////
@@ -245,57 +261,63 @@ QString PQCLoadImage::load(QString filename, QSize requestedSize, QSize &origSiz
 
     // resvg trumps Qt's SVG engine
 #ifdef PQMRESVG
-    if(PQCImageFormats::get().getEnabledFormatsResvg().contains(suffix))
+    if(PQCImageFormats::get().getEnabledFormatsResvg().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsResvg().contains(suffix2))
         err = PQCLoadImageResvg::load(filename, requestedSize, origSize, img);
 #endif
 
 #ifdef PQMPOPPLER
-    if(img.isNull() && PQCImageFormats::get().getEnabledFormatsPoppler().contains(suffix))
+    if(img.isNull() && (PQCImageFormats::get().getEnabledFormatsPoppler().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsPoppler().contains(suffix2)))
         err = PQCLoadImagePoppler::load(filename, requestedSize, origSize, img);
 #endif
 
 #ifdef PQMQTPDF
-    if(img.isNull() && PQCImageFormats::get().getEnabledFormatsPoppler().contains(suffix))
+    if(img.isNull() && (PQCImageFormats::get().getEnabledFormatsPoppler().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsPoppler().contains(suffix2)))
         err = PQCLoadImageQtPDF::load(filename, requestedSize, origSize, img);
 #endif
 
-    if(img.isNull() && PQCImageFormats::get().getEnabledFormatsQt().contains(suffix))
+    if(img.isNull() && (PQCImageFormats::get().getEnabledFormatsQt().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsQt().contains(suffix2)))
         err = PQCLoadImageQt::load(filename, requestedSize, origSize, img);
 
 #ifdef PQMRAW
-    if(img.isNull() && PQCImageFormats::get().getEnabledFormatsLibRaw().contains(suffix))
+    if(img.isNull() && (PQCImageFormats::get().getEnabledFormatsLibRaw().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsLibRaw().contains(suffix2)))
         err = PQCLoadImageRAW::load(filename, requestedSize, origSize, img);
 #endif
 
 #ifdef PQMLIBARCHIVE
-    if(img.isNull() && PQCImageFormats::get().getEnabledFormatsLibArchive().contains(suffix))
+    if(img.isNull() && (PQCImageFormats::get().getEnabledFormatsLibArchive().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsLibArchive().contains(suffix2)))
         err = PQCLoadImageArchive::load(filename, requestedSize, origSize, img);
 #endif
 
-    if(img.isNull() && PQCImageFormats::get().getEnabledFormatsXCFTools().contains(suffix))
+#ifdef PQMLIBSAI
+    if(img.isNull() && (PQCImageFormats::get().getEnabledFormatsLibsai().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsLibsai().contains(suffix2)))
+        err = PQCLoadImageLibsai::load(filename, requestedSize, origSize, img);
+#endif
+
+    if(img.isNull() && (PQCImageFormats::get().getEnabledFormatsXCFTools().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsXCFTools().contains(suffix2)))
         err = PQCLoadImageXCF::load(filename, requestedSize, origSize, img);
 
 #if defined(PQMIMAGEMAGICK) || defined(PQMGRAPHICSMAGICK)
-    if(img.isNull() && PQCImageFormats::get().getEnabledFormatsMagick().contains(suffix))
+    if(img.isNull() && (PQCImageFormats::get().getEnabledFormatsMagick().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsMagick().contains(suffix2)))
         err = PQCLoadImageMagick::load(filename, requestedSize, origSize, img);
 #endif
 
 #ifdef PQMLIBVIPS
-    if((err != "" || img.isNull()) && PQCImageFormats::get().getEnabledFormatsLibVips().contains(suffix))
+    if((err != "" || img.isNull()) && (PQCImageFormats::get().getEnabledFormatsLibVips().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsLibVips().contains(suffix2)))
         err = PQCLoadImageLibVips::load(filename, requestedSize, origSize, img);
 #endif
 
 #ifdef PQMFREEIMAGE
-    if(img.isNull() && PQCImageFormats::get().getEnabledFormatsFreeImage().contains(suffix))
+    if(img.isNull() && (PQCImageFormats::get().getEnabledFormatsFreeImage().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsFreeImage().contains(suffix2)))
         err = PQCLoadImageFreeImage::load(filename, requestedSize, origSize, img);
 #endif
 
 #ifdef PQMDEVIL
-    if(img.isNull() && PQCImageFormats::get().getEnabledFormatsDevIL().contains(suffix))
+    if(img.isNull() && (PQCImageFormats::get().getEnabledFormatsDevIL().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsDevIL().contains(suffix2)))
         err = PQCLoadImageDevil::load(filename, requestedSize, origSize, img);
 #endif
 
-    if(img.isNull() && (PQCImageFormats::get().getEnabledFormatsVideo().contains(suffix) || PQCImageFormats::get().getEnabledFormatsLibmpv().contains(suffix)))
+    if(img.isNull() && (PQCImageFormats::get().getEnabledFormatsVideo().contains(suffix1) || PQCImageFormats::get().getEnabledFormatsLibmpv().contains(suffix1) ||
+                        PQCImageFormats::get().getEnabledFormatsVideo().contains(suffix2) || PQCImageFormats::get().getEnabledFormatsLibmpv().contains(suffix2)))
         err = PQCLoadImageVideo::load(filename, requestedSize, origSize, img);
 
 
@@ -311,7 +333,7 @@ QString PQCLoadImage::load(QString filename, QSize requestedSize, QSize &origSiz
 
             // resvg trumps Qt's SVG engine
 #ifdef PQMRESVG
-            if(PQCImageFormats::get().getEnabledMimeTypesResvg().contains(suffix))
+            if(PQCImageFormats::get().getEnabledMimeTypesResvg().contains(mimetype))
                 err = PQCLoadImageResvg::load(filename, requestedSize, origSize, img);
 #endif
 
@@ -338,6 +360,11 @@ QString PQCLoadImage::load(QString filename, QSize requestedSize, QSize &origSiz
                 err = PQCLoadImageArchive::load(filename, requestedSize, origSize, img);
 #endif
 
+#ifdef PQMLIBSAI
+            if(img.isNull() && PQCImageFormats::get().getEnabledMimeTypesLibsai().contains(mimetype))
+                err = PQCLoadImageLibsai::load(filename, requestedSize, origSize, img);
+#endif
+
             if(img.isNull() && PQCImageFormats::get().getEnabledMimeTypesXCFTools().contains(mimetype))
                 err = PQCLoadImageXCF::load(filename, requestedSize, origSize, img);
 
@@ -372,7 +399,8 @@ QString PQCLoadImage::load(QString filename, QSize requestedSize, QSize &origSiz
 #if defined(PQMGRAPHICSMAGICK) || defined(PQMIMAGEMAGICK)
     // if everything failed, we make sure to try one more time with ImageMagick or GraphicsMagick to see what could be done
     // we do not do this for video files as it can lead to resource intensive ffmpeg processes that may persist after PhotoQt is closed
-    if(img.isNull() && !PQCImageFormats::get().getEnabledFormatsVideo().contains(suffix) && !PQCImageFormats::get().getEnabledFormatsLibmpv().contains(suffix)) {
+    if(img.isNull() && !PQCImageFormats::get().getEnabledFormatsVideo().contains(suffix1) && !PQCImageFormats::get().getEnabledFormatsLibmpv().contains(suffix1) &&
+                       !PQCImageFormats::get().getEnabledFormatsVideo().contains(suffix2) && !PQCImageFormats::get().getEnabledFormatsLibmpv().contains(suffix2)) {
 
         qDebug() << "null image, try magick";
 
