@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -23,6 +23,8 @@
 #include <QtDebug>
 #include <QFileInfo>
 #include <pqc_commandlineparser.h>
+#include<QDir>
+#include <iostream>
 
 PQCCommandLineParser::PQCCommandLineParser(QApplication &app, QObject *parent) : QObject(parent), QCommandLineParser() {
 
@@ -81,7 +83,18 @@ PQCCommandLineParser::PQCCommandLineParser(QApplication &app, QObject *parent) :
                    //: Command line option
         {"reset-config", QApplication::translate("commandlineparser", "Reset default configuration.")},
                    //: Command line option
-        {"show-info", QApplication::translate("commandlineparser", "Show configuration overview.")}
+        {"show-info", QApplication::translate("commandlineparser", "Show configuration overview.")},
+                   //: Command line option
+        {"modern", QApplication::translate("commandlineparser", "Launch with modern interface.")},
+                   //: Command line option
+        {"integrated", QApplication::translate("commandlineparser", "Launch with integrated interface.")},
+                   //: Command line option
+        {"show-wizard", QApplication::translate("commandlineparser", "Show the startup wizard.")},
+                   //: Command line option
+        {"skip-wizard", QApplication::translate("commandlineparser", "Don't show the startup wizard.")},
+                   //: Command line option
+        {"install-extension", QApplication::translate("commandlineparser", "Install specified extension."),
+                   "extension.pqe"},
     });
 
     process(app);
@@ -92,10 +105,37 @@ PQCCommandLineResult PQCCommandLineParser::getResult() {
 
     PQCCommandLineResult ret = PQCCommandLineNothing;
 
+#if defined(Q_OS_WIN) && defined(PQMPORTABLETWEAKS)
+    // the first entry is the config dir that needs to be ignored
+    if(positionalArguments().length() > 1) {
+        filenames.clear();
+        // the first entry is the config dir and needs to be ignored
+        bool first = true;
+        for(auto &f : positionalArguments()) {
+            if(first) {
+                first = false;
+                continue;
+            }
+            // if we have portable tweaks enabled on Windows we need to ignore any executable that might be part of the command line
+            if(!f.endsWith(".exe")) {
+                // we also need to check if it is a relative path, as we want to use the portable dir and NOT the (temporary) exe dir
+                QFileInfo info(f);
+                if(info.isRelative())
+                    filenames.append(qgetenv("PHOTOQT_PORTABLE_DATA_LOCATION") + "/../" + info.filePath());
+                else
+                    filenames.append(f);
+            }
+        }
+        if(filenames.length())
+            ret = ret|PQCCommandLineFile;
+
+    }
+#else
     if(positionalArguments().length() > 0) {
         ret = ret|PQCCommandLineFile;
         filenames = positionalArguments();
     }
+#endif
 
     if(isSet("o") || isSet("open"))
         ret = ret|PQCCommandLineOpen;
@@ -148,15 +188,31 @@ PQCCommandLineResult PQCCommandLineParser::getResult() {
     if(isSet("show-info"))
         ret = ret|PQCCommandLineShowInfo;
 
+    if(isSet("modern"))
+        ret = ret|PQCCommandLineModernInterface;
+
+    if(isSet("integrated"))
+        ret = ret|PQCCommandLineIntegratedInterface;
+
     if(isSet("setting")) {
         const QStringList tmp = value("setting").split(":");
         if(tmp.length() == 2) {
             ret = ret|PQCCommandLineSettingUpdate;
-            settingUpdate = tmp;
+            settingUpdate[0] = tmp[0];
+            settingUpdate[1] = tmp[1];
         }
-
     }
 
+    if(isSet("show-wizard"))
+        ret = ret|PQCCommandLineShowStartupWizard;
+
+    if(isSet("skip-wizard"))
+        ret = ret|PQCCommandLineDontShowStartupWizard;
+
+    extensionFileName = value("install-extension");
+    if(extensionFileName != "")
+        ret = ret|PQCCommandLineInstallExtension;
+
     return ret;
 
 }
