@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2024 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -21,18 +21,25 @@
  **************************************************************************/
 
 #include <pqc_photosphere.h>
+
+#ifdef PQMPHOTOSPHERE
 #include <pqc_photosphererenderer.h>
 #include <pqc_configfiles.h>
+#include <pqc_loadimage.h>
 
 #include <QSGNode>
-#include <QFile>
-#include <QFileInfo>
+#include <QImage>
+#include <QBuffer>
+#include <QOpenGLContext>
 
 #ifdef PQMEXIV2
 #include <exiv2/exiv2.hpp>
 #endif
 
 PQCPhotoSphere::PQCPhotoSphere(QQuickItem *parent) : QQuickFramebufferObject(parent), recreateRenderer(false) {
+#else
+PQCPhotoSphere::PQCPhotoSphere(QQuickItem *parent) : QQuickItem(parent) {
+#endif
 
     m_azimuth = 180;
     m_elevation = 0;
@@ -40,9 +47,15 @@ PQCPhotoSphere::PQCPhotoSphere(QQuickItem *parent) : QQuickFramebufferObject(par
 
     partial = false;
 
+    // this will be set to false once we know that the necessary OpenGL is not supported
+    // see updatePaintNode() below
+    m_isSupported = true;
+
+#ifdef PQMPHOTOSPHERE
     setFlag(ItemHasContents);
     setTextureFollowsItemSize(true);
     setMirrorVertically(true);
+#endif
 
 }
 
@@ -62,8 +75,10 @@ void PQCPhotoSphere::setAzimuth(double azimuth) {
     if(azimuth == m_azimuth)
         return;
 
+#ifdef PQMPHOTOSPHERE
     m_azimuth = azimuth;
     updateSphere();
+#endif
     Q_EMIT azimuthChanged();
 
 }
@@ -77,8 +92,10 @@ void PQCPhotoSphere::setElevation(double elevation) {
     if(elevation == m_elevation || !qIsFinite(elevation))
         return;
 
+#ifdef PQMPHOTOSPHERE
     m_elevation = qBound<double>(-90.0, elevation, 90.0);
     updateSphere();
+#endif
     Q_EMIT elevationChanged();
 
 }
@@ -92,8 +109,11 @@ void PQCPhotoSphere::setFieldOfView(double fieldOfView) {
     if(fieldOfView == m_fieldOfView || fieldOfView < 3.0 || fieldOfView > 150.0)
         return;
 
+#ifdef PQMPHOTOSPHERE
     m_fieldOfView = fieldOfView;
     updateSphere();
+#endif
+
     Q_EMIT fieldOfViewChanged();
 
 }
@@ -104,6 +124,8 @@ QString PQCPhotoSphere::getSource() {
 
 void PQCPhotoSphere::setSource(QString path) {
 
+#ifdef PQMPHOTOSPHERE
+
     path = QUrl::fromPercentEncoding(path.toUtf8());
 
     if(path == "")
@@ -116,13 +138,18 @@ void PQCPhotoSphere::setSource(QString path) {
             qWarning() << "Unable to open file.";
             return;
         }
-        QFileInfo info(path);
-        QDataStream in(&file);
-        QByteArray data(info.size(), 0);
-        in.readRawData(data.data(), info.size());
+
+        // load full image and set as raw data
+
+        QSize s;
+        QImage img;
+        PQCLoadImage::get().load(path, QSize(), s, img);
+
+        QBuffer buffer(&image);
+        buffer.open(QIODevice::WriteOnly);
+        img.save(&buffer, "JPG");
 
         m_imageUrl = path;
-        image = data;
 
 #if defined(PQMEXIV2) && defined(PQMEXIV2_ENABLE_BMFF)
 
@@ -195,6 +222,8 @@ void PQCPhotoSphere::setSource(QString path) {
 
     recreateRenderer = true;
 
+#endif
+
 }
 
 
@@ -214,8 +243,18 @@ QSize PQCPhotoSphere::getFullSize() {
     return fullSize;
 }
 
+#ifdef PQMPHOTOSPHERE
 QSGNode *PQCPhotoSphere::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *updatePaintNodeData) {
 
+    // ensure that the necessary OpenGL is available
+    // if not, then we set this variable to false
+    // which will be reacted to in PQPhotoSphere
+    QOpenGLContext *context = QOpenGLContext::currentContext();
+    if(!context && m_isSupported) {
+        m_isSupported = false;
+        Q_EMIT isSupportedChanged();
+    }
+
     if(oldNode && recreateRenderer) {
         delete oldNode;
         oldNode = nullptr;
@@ -235,3 +274,4 @@ void PQCPhotoSphere::updateSphere() {
     polish();
     update();
 }
+#endif
