@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -20,6 +20,8 @@
  **                                                                      **
  **************************************************************************/
 
+#ifdef PQMPHOTOSPHERE
+
 #include "pqc_photosphereitem.h"
 
 #include <QOpenGLFunctions>
@@ -143,3 +145,5 @@ void PQCPhotoSphereItem::drawSphere() {
     QOpenGLVertexArrayObject::Binder vertexArrowBinderBinder(&vertexArrowObject);
     QOpenGLContext::currentContext()->functions()->glDrawArrays(GL_TRIANGLES, 0, sphereVertices.size());
 }
+
+#endif
