@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -27,16 +27,17 @@
 #include <QMimeDatabase>
 #include <QSize>
 #include <QTimer>
-#include <QtConcurrent>
+#include <QtConcurrent/QtConcurrentRun>
 #include <QtDebug>
+#include <QImage>
+#include <QDirIterator>
 #include <pqc_configfiles.h>
 #include <pqc_filefoldermodel.h>
 #include <pqc_imageformats.h>
 #include <pqc_loadimage.h>
-#include <pqc_notify.h>
+#include <pqc_notify_cpp.h>
 #include <pqc_resolutioncache.h>
-#include <pqc_settings.h>
-#include <scripts/pqc_scriptsfiledialog.h>
+#include <pqc_settingscpp.h>
 #include <scripts/pqc_scriptsimages.h>
 
 #ifdef PQMLIBARCHIVE
@@ -56,13 +57,10 @@
 #include <exiv2/exiv2.hpp>
 #endif
 
-PQCFileFolderModel &PQCFileFolderModel::get() {
-    static PQCFileFolderModel instance;
-    return instance;
-}
-
 PQCFileFolderModel::PQCFileFolderModel(QObject *parent) : QObject(parent) {
 
+    m_firstFolderMainViewLoaded = false;
+
     m_fileInFolderMainView = "";
     m_folderFileDialog = "";
     m_countMainView = 0;
@@ -72,6 +70,9 @@ PQCFileFolderModel::PQCFileFolderModel(QObject *parent) : QObject(parent) {
     m_readDocumentOnly = false;
     m_readArchiveOnly = false;
     m_includeFilesInSubFolders = false;
+    m_isARC = false;
+    m_isPDF = false;
+    m_activeViewerMode = false;
 
     m_entriesMainView.clear();
     m_entriesFileDialog.clear();
@@ -82,6 +83,12 @@ PQCFileFolderModel::PQCFileFolderModel(QObject *parent) : QObject(parent) {
     m_restrictToMimeTypes = PQCImageFormats::get().getEnabledMimeTypes();
     m_imageResolutionFilter = QSize(0,0);
     m_fileSizeFilter = 0;
+    m_justLeftViewerMode = false;
+
+    m_virtualFolders = PQCNotifyCPP::get().getVirtualFolders();
+    m_virtualFiles = PQCNotifyCPP::get().getVirtualFiles();
+    m_loadVirtualFolderFileDialog = (m_virtualFolders.length()+m_virtualFiles.length() > 0);
+    m_loadVirtualFolderMainView = (m_virtualFolders.length()+m_virtualFiles.length() > 0);
 
     watcherMainView = new QFileSystemWatcher;
     watcherFileDialog = new QFileSystemWatcher;
@@ -131,18 +138,45 @@ PQCFileFolderModel::PQCFileFolderModel(QObject *parent) : QObject(parent) {
         }
     });
 
+    timerResetJustLeftViewerMode = new QTimer;
+    timerResetJustLeftViewerMode->setInterval(100);
+    timerResetJustLeftViewerMode->setSingleShot(true);
+    connect(timerResetJustLeftViewerMode, &QTimer::timeout, this, [=]() {
+        m_justLeftViewerMode = false;
+    });
+
     // we add a tiny delay to this signal to make sure that when the directory has changed all files are fully written
     // not having this delay can cause faulty thumbnails to be loaded
-    connect(watcherMainView, &QFileSystemWatcher::directoryChanged, this, [=]() { m_fileInFolderMainView = m_currentFile; loadDelayMainView->start(); });
-    connect(watcherFileDialog, &QFileSystemWatcher::directoryChanged, this, [=]() { loadDelayFileDialog->start(); });
+    connect(watcherMainView, &QFileSystemWatcher::directoryChanged, this, [=]() { /*if(m_ignoreWatcherTemporarily) return;*/ m_fileInFolderMainView = m_currentFile; loadDelayMainView->start(); });
+    connect(watcherFileDialog, &QFileSystemWatcher::directoryChanged, this, [=]() { /*if(m_ignoreWatcherTemporarily) return;*/  loadDelayFileDialog->start(); });
 
     m_advancedSortDone = 0;
 
     connect(this, &PQCFileFolderModel::newDataLoadedMainView, this, &PQCFileFolderModel::handleNewDataLoadedMainView);
 
-    connect(&PQCSettings::get(), &PQCSettings::valueChanged, this, &PQCFileFolderModel::handleSettingsChanges);
+    connect(&PQCSettingsCPP::get(), &PQCSettingsCPP::imageviewSortImagesAscendingChanged, this, &PQCFileFolderModel::loadDataFileDialog);
+    connect(&PQCSettingsCPP::get(), &PQCSettingsCPP::imageviewSortImagesAscendingChanged, this, &PQCFileFolderModel::loadDataMainView);
+    connect(&PQCSettingsCPP::get(), &PQCSettingsCPP::imageviewSortImagesByChanged, this, &PQCFileFolderModel::loadDataFileDialog);
+    connect(&PQCSettingsCPP::get(), &PQCSettingsCPP::imageviewSortImagesByChanged, this, &PQCFileFolderModel::loadDataMainView);
+    connect(&PQCSettingsCPP::get(), &PQCSettingsCPP::filedialogShowHiddenFilesFoldersChanged, this, &PQCFileFolderModel::loadDataFileDialog);
+    connect(&PQCSettingsCPP::get(), &PQCSettingsCPP::filedialogShowHiddenFilesFoldersChanged, this, &PQCFileFolderModel::loadDataMainView);
 
-    connect(&PQCNotify::get(), &PQCNotify::resetSessionData, this, &PQCFileFolderModel::resetModel);
+    connect(&PQCNotifyCPP::get(), &PQCNotifyCPP::resetSessionData, this, &PQCFileFolderModel::resetModel);
+
+    connect(&PQCNotifyCPP::get(), &PQCNotifyCPP::virtualFoldersChanged,
+            this, [=](QStringList val) { m_virtualFolders = val; Q_EMIT virtualFoldersChanged(); });
+    connect(&PQCNotifyCPP::get(), &PQCNotifyCPP::virtualFilesChanged,
+            this, [=](QStringList val) { m_virtualFiles = val; Q_EMIT virtualFilesChanged(); });
+
+    /********************************************/
+    /********************************************/
+
+    // these are READ FROM PQCFileFolderModelCPP
+
+    connect(&PQCFileFolderModelCPP::get(), &PQCFileFolderModelCPP::setFileInFolderMainView, this, [=](QString val) { setFileInFolderMainView(val); });
+
+    /********************************************/
+    /********************************************/
 
 }
 
@@ -154,14 +188,8 @@ PQCFileFolderModel::~PQCFileFolderModel() {
     delete watcherMainView;
     delete watcherFileDialog;
 
-}
-
-void PQCFileFolderModel::handleSettingsChanges(const QString &key, const QVariant &value) {
-
-    if(key == "imageviewSortImagesAscending" || key == "imageviewSortImagesBy" || key == "filedialogShowHiddenFilesFolders") {
-        loadDataFileDialog();
-        loadDataMainView();
-    }
+    delete timerNotifyCurrentIndexChanged;
+    delete timerResetJustLeftViewerMode;
 
 }
 
@@ -179,6 +207,7 @@ void PQCFileFolderModel::setFileInFolderMainView(QString val) {
     QFileInfo newfile(val);
     if(oldfile.dir() == newfile.dir() && m_fileInFolderMainView != "") {
         m_currentFile = val;
+        PQCFileFolderModelCPP::get().setCurrentFile(m_currentFile);
         m_currentFileNoDelay = m_currentFile;
         m_currentIndex = m_entriesMainView.indexOf(val);
         m_currentIndexNoDelay = m_currentIndex;
@@ -205,11 +234,27 @@ void PQCFileFolderModel::setFolderFileDialog(QString val) {
     Q_EMIT folderFileDialogChanged();
     loadDelayFileDialog->start();
 
-    if(val != "")
-        PQCScriptsFileDialog::get().setLastLocation(val);
+    if(val != "") {
+        QFile file(PQCConfigFiles::get().FILEDIALOG_LAST_LOCATION());
+        if(file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
+            QTextStream out(&file);
+            out << val;
+            file.close();
+        }
+    }
 
 }
 
+bool PQCFileFolderModel::getFirstFolderMainViewLoaded() {
+    return m_firstFolderMainViewLoaded;
+}
+void PQCFileFolderModel::setFirstFolderMainViewLoaded(bool val) {
+    if(val != m_firstFolderMainViewLoaded) {
+        m_firstFolderMainViewLoaded = val;
+        Q_EMIT firstFolderMainViewLoadedChanged();
+    }
+}
+
 /********************************************/
 /********************************************/
 
@@ -217,6 +262,9 @@ int PQCFileFolderModel::getCountMainView() {
     return m_countMainView;
 }
 void PQCFileFolderModel::setCountMainView(int c) {
+
+    PQCFileFolderModelCPP::get().setCountMainView(c);
+
     if(m_countMainView == c)
         return;
     m_countMainView = c;
@@ -295,13 +343,36 @@ void PQCFileFolderModel::setReadArchiveOnly(bool c) {
     loadDelayMainView->start();
 }
 
-QStringList PQCFileFolderModel::getExtraFoldersToLoad() {
-    return m_extraFoldersToLoad;
+QStringList PQCFileFolderModel::getVirtualFolders() {
+    return m_virtualFolders;
 }
-void PQCFileFolderModel::setExtraFoldersToLoad(QStringList val) {
-    m_extraFoldersToLoad = val;
-    Q_EMIT extraFoldersToLoadChanged();
-    loadDelayMainView->start();
+void PQCFileFolderModel::setVirtualFolders(QStringList val) {
+    m_virtualFolders = val;
+    Q_EMIT virtualFoldersChanged();
+}
+
+QStringList PQCFileFolderModel::getVirtualFiles() {
+    return m_virtualFiles;
+}
+void PQCFileFolderModel::setVirtualFiles(QStringList val) {
+    m_virtualFiles = val;
+    Q_EMIT virtualFilesChanged();
+}
+
+bool PQCFileFolderModel::getLoadVirtualFolderFileDialog() {
+    return m_loadVirtualFolderFileDialog;
+}
+void PQCFileFolderModel::setLoadVirtualFolderFileDialog(bool val) {
+    m_loadVirtualFolderFileDialog = val;
+    Q_EMIT loadVirtualFolderFileDialogChanged();
+}
+
+bool PQCFileFolderModel::getLoadVirtualFolderMainView() {
+    return m_loadVirtualFolderMainView;
+}
+void PQCFileFolderModel::setLoadVirtualFolderMainView(bool val) {
+    m_loadVirtualFolderMainView = val;
+    Q_EMIT loadVirtualFolderMainViewChanged();
 }
 
 /********************************************/
@@ -418,27 +489,26 @@ int PQCFileFolderModel::getAdvancedSortDone() {
 
 /********************************************/
 
-void PQCFileFolderModel::advancedSortMainView() {
+void PQCFileFolderModel::advancedSortMainView(QString advSortCriteria, bool advSortAscending, QString advSortQuality, QStringList advDateCriteria) {
 
     qDebug() << "";
 
     // if nothing changed, reload folder
     QFileInfo info(m_fileInFolderMainView);
     if(info.absolutePath() == cacheAdvancedSortFolderName
-        && PQCSettings::get()["imageviewAdvancedSortCriteria"].toString() == cacheAdvancedSortCriteria
+        && advSortCriteria == cacheAdvancedSortCriteria
         && info.lastModified().toMSecsSinceEpoch() == cacheAdvancedSortLastModified
-        && PQCSettings::get()["imageviewAdvancedSortAscending"].toBool() == cacheAdvancedSortAscending) {
+        && advSortAscending == cacheAdvancedSortAscending) {
 
         // we first make sure the count is set to 0
         // to force a refresh of the folder
-        const int tmp = m_countMainView;
-        m_countMainView = 0;
-        Q_EMIT countMainViewChanged();
-        m_countMainView = tmp;
+        const int tmp = getCountMainView();
+        setCountMainView(0);
+        setCountMainView(tmp);
 
         m_entriesMainView = cacheAdvancedSortFolder;
+        PQCFileFolderModelCPP::get().setEntriesMainView(cacheAdvancedSortFolder);
         Q_EMIT newDataLoadedMainView();
-        Q_EMIT countMainViewChanged();
         Q_EMIT advancedSortingComplete();
         return;
 
@@ -464,7 +534,7 @@ void PQCFileFolderModel::advancedSortMainView() {
             // depending on the criteria, it is computed in different ways
             qint64 key = 0;
 
-            if(PQCSettings::get()["imageviewAdvancedSortCriteria"].toString() == "resolution") {
+            if(advSortCriteria == "resolution") {
 
                 QSize size = PQCResolutionCache::get().getResolution(fn);
                 if(!size.isValid()) {
@@ -475,12 +545,12 @@ void PQCFileFolderModel::advancedSortMainView() {
 
                 key = size.width()+size.height();
 
-            } else if(PQCSettings::get()["imageviewAdvancedSortCriteria"].toString() == "luminosity") {
+            } else if(advSortCriteria == "luminosity") {
 
                 QSize requestedSize = QSize(512,512);
-                if(PQCSettings::get()["imageviewAdvancedSortQuality"].toString() == "medium")
+                if(advSortQuality == "medium")
                     requestedSize = QSize(1024,1024);
-                else if(PQCSettings::get()["imageviewAdvancedSortQuality"].toString() == "high")
+                else if(advSortQuality == "high")
                     requestedSize = QSize(-1,-1);
 
                 QSize origSize;
@@ -514,15 +584,13 @@ void PQCFileFolderModel::advancedSortMainView() {
 
                 key = val;
 
-            } else if(PQCSettings::get()["imageviewAdvancedSortCriteria"].toString() == "exifdate") {
-
-                QStringList order = PQCSettings::get()["imageviewAdvancedSortDateCriteria"].toStringList();
+            } else if(advSortCriteria == "exifdate") {
 
                 bool foundvalue = false;
 
-                for(int j = 0; j < order.length(); ++j) {
+                for(int j = 0; j < advDateCriteria.length(); ++j) {
 
-                    QString item = order.at(j);
+                    QString item = advDateCriteria.at(j);
 
                     if(item == "exiforiginal" || item == "exifdigital") {
 
@@ -626,9 +694,9 @@ void PQCFileFolderModel::advancedSortMainView() {
             } else {
 
                 QSize requestedSize = QSize(512,512);
-                if(PQCSettings::get()["imageviewAdvancedSortQuality"].toString() == "medium")
+                if(advSortQuality == "medium")
                     requestedSize = QSize(1024,1024);
-                else if(PQCSettings::get()["imageviewAdvancedSortQuality"].toString() == "high")
+                else if(advSortQuality == "high")
                     requestedSize = QSize(-1,-1);
 
                 QSize origSize;
@@ -674,7 +742,7 @@ void PQCFileFolderModel::advancedSortMainView() {
                 qint64 green_val = 0;
                 qint64 blue_val = 0;
 
-                if(PQCSettings::get()["imageviewAdvancedSortCriteria"].toString() == "dominantcolor") {
+                if(advSortCriteria == "dominantcolor") {
 
                     QVector<qint64> redSteps(26);
                     QVector<qint64> greenSteps(26);
@@ -716,7 +784,7 @@ void PQCFileFolderModel::advancedSortMainView() {
         if(!advancedSortKeepGoing) return;
 
         QList<qint64> allKeys = sortedWithKey.keys();
-        if(PQCSettings::get()["imageviewAdvancedSortAscending"].toBool())
+        if(advSortAscending)
             std::sort(allKeys.begin(), allKeys.end(), std::less<int>());
         else
             std::sort(allKeys.begin(), allKeys.end(), std::greater<int>());
@@ -725,7 +793,7 @@ void PQCFileFolderModel::advancedSortMainView() {
         for(auto entry : std::as_const(allKeys)) {
             QStringList curVals = sortedWithKey[entry];
             curVals.sort(Qt::CaseInsensitive);
-            if(!PQCSettings::get()["imageviewAdvancedSortAscending"].toBool())
+            if(!advSortAscending)
                 std::reverse(curVals.begin(), curVals.end());
             for(const auto &e : std::as_const(curVals))
                 cacheAdvancedSortFolder << e;
@@ -733,16 +801,15 @@ void PQCFileFolderModel::advancedSortMainView() {
 
         // we first make sure the count is set to 0
         // to force a refresh of the folder
-        const int tmp = m_countMainView;
-        m_countMainView = 0;
-        Q_EMIT countMainViewChanged();
-        m_countMainView = tmp;
+        const int tmp = getCountMainView();
+        setCountMainView(0);
+        setCountMainView(tmp);
 
         m_currentIndex = cacheAdvancedSortFolder.indexOf(m_currentFile);
         m_currentIndexNoDelay = m_currentIndex;
 
         m_entriesMainView = cacheAdvancedSortFolder;
-        Q_EMIT countMainViewChanged();
+        PQCFileFolderModelCPP::get().setEntriesMainView(cacheAdvancedSortFolder);
         Q_EMIT currentIndexChanged();
         Q_EMIT currentIndexNoDelayChanged();
         Q_EMIT newDataLoadedMainView();
@@ -752,8 +819,8 @@ void PQCFileFolderModel::advancedSortMainView() {
         QFileInfo info(m_fileInFolderMainView);
         cacheAdvancedSortFolderName = info.absolutePath();
         cacheAdvancedSortLastModified = info.lastModified().toMSecsSinceEpoch();
-        cacheAdvancedSortCriteria = PQCSettings::get()["imageviewAdvancedSortCriteria"].toString();
-        cacheAdvancedSortAscending = PQCSettings::get()["imageviewAdvancedSortAscending"].toBool();
+        cacheAdvancedSortCriteria = advSortCriteria;
+        cacheAdvancedSortAscending = advSortAscending;
 
     });
 
@@ -809,6 +876,7 @@ void PQCFileFolderModel::resetModel() {
     delete watcherMainView;
     watcherMainView = new QFileSystemWatcher;
     m_entriesMainView.clear();
+    PQCFileFolderModelCPP::get().setEntriesMainView({});
 
     setFileInFolderMainView("");
     setCountMainView(0);
@@ -818,6 +886,7 @@ void PQCFileFolderModel::resetModel() {
     m_includeFilesInSubFolders = false;
 
     m_entriesMainView.clear();
+    PQCFileFolderModelCPP::get().setEntriesMainView({});
 
     cache.resetData();
 
@@ -839,12 +908,15 @@ int PQCFileFolderModel::getCurrentIndexNoDelay() {
 
 void PQCFileFolderModel::setCurrentIndex(int val) {
 
+    PQCFileFolderModelCPP::get().setCurrentIndex(val);
+
     if(m_currentIndex != val) {
         m_currentIndex = val;
         if(m_currentIndex == -1)
             m_currentFile = "";
         else
             m_currentFile = m_entriesMainView[m_currentIndex];
+        PQCFileFolderModelCPP::get().setCurrentFile(m_currentFile);
 
         m_currentIndexNoDelay = m_currentIndex;
         m_currentFileNoDelay = m_currentFile;
@@ -859,6 +931,10 @@ void PQCFileFolderModel::setCurrentIndex(int val) {
 
 }
 
+int PQCFileFolderModel::getIndexOf(QString file) {
+    return m_entriesMainView.indexOf(file);
+}
+
 QString PQCFileFolderModel::getCurrentFile() {
     return m_currentFile;
 }
@@ -900,18 +976,53 @@ void PQCFileFolderModel::loadDataMainView() {
     qDebug() << "";
 
     ////////////////////////
-    // clear old entries
+    // virtual folder
+
+    bool virtualFolderLoaded = false;
+    bool virtualFolderEmpty = false;
+    if(m_loadVirtualFolderMainView) {
+
+        qDebug() << "Loading virtual folder:" << m_virtualFolders << "/" << m_virtualFiles;
+
+        QStringList allFiles = m_virtualFiles;
+        for(const QString &f : std::as_const(m_virtualFolders)) {
+
+            allFiles.append(getAllFiles(f));
+
+        }
+
+        m_entriesMainView = allFiles;
+
+        PQCFileFolderModelCPP::get().setEntriesMainView(m_entriesMainView);
+        setCountMainView(m_entriesMainView.length());
+
+        if(allFiles.length() == 0)
+            virtualFolderEmpty = true;
+        else
+            virtualFolderLoaded = true;
+
+    }
 
-    m_entriesMainView.clear();
-    m_countMainView = 0;
-    delete watcherMainView;
-    watcherMainView = new QFileSystemWatcher;
 
     ////////////////////////
     // no new directory
 
-    if(m_fileInFolderMainView.isEmpty()) {
+    if(virtualFolderEmpty || m_fileInFolderMainView.isEmpty()) {
+
+        qDebug() << "No file in main view -> nothing here";
+
+        // clear old entries
+        m_entriesMainView.clear();
+        PQCFileFolderModelCPP::get().setEntriesMainView({});
+        if(watcherMainView->directories().length())
+            watcherMainView->removePaths(watcherMainView->directories());
+        if(watcherMainView->files().length())
+            watcherMainView->removePaths(watcherMainView->files());
+        setCountMainView(0);
+
+        // reset variables
         m_currentFile = "";
+        PQCFileFolderModelCPP::get().setCurrentFile(m_currentFile);
         m_currentIndex = -1;
         m_currentIndexNoDelay = -1;
         m_currentFileNoDelay = "";
@@ -921,59 +1032,84 @@ void PQCFileFolderModel::loadDataMainView() {
         Q_EMIT currentIndexChanged();
         Q_EMIT currentFileNoDelayChanged();
         Q_EMIT currentIndexNoDelayChanged();
+        if(!m_firstFolderMainViewLoaded)
+            setFirstFolderMainViewLoaded(true);
         return;
     }
 
-    ////////////////////////
-    // load files
+    if(!virtualFolderLoaded) {
 
-    int numberPageDocument = 0;
-    if(m_fileInFolderMainView.contains("::PDF::")) {
-        m_readDocumentOnly = true;
-        numberPageDocument = m_fileInFolderMainView.split("::PDF::").at(0).toInt();
-        m_fileInFolderMainView = m_fileInFolderMainView.split("::PDF::").at(1);
-    } else if(m_fileInFolderMainView.contains("::ARC::")) {
-        m_readArchiveOnly = true;
-        numberPageDocument = m_fileInFolderMainView.split("::ARC::").at(0).toInt();
-        m_fileInFolderMainView = m_fileInFolderMainView.split("::ARC::").at(1);
-    }
+        ////////////////////////
+        // load files
 
-    const bool isFolder = !QFileInfo(m_fileInFolderMainView).isFile();
-
-    watcherMainView->addPath(isFolder ? m_fileInFolderMainView : QFileInfo(m_fileInFolderMainView).absolutePath());
-    connect(watcherMainView, &QFileSystemWatcher::directoryChanged, this, [=]() { m_fileInFolderMainView = m_currentFile; loadDelayMainView->start(); });
-
-    if(m_readDocumentOnly) {// && PQCImageFormats::get().getEnabledFormatsPoppler().contains(QFileInfo(m_fileInFolderMainView).suffix().toLower())) {
-
-        m_entriesMainView = listPDFPages(m_fileInFolderMainView);
-        m_countMainView = m_entriesMainView.length();
-        m_readDocumentOnly = false;
-        setCurrentIndex(numberPageDocument);
-
-    } else if(m_readArchiveOnly) {// && PQCImageFormats::get().getEnabledFormatsLibArchive().contains(QFileInfo(m_fileInFolderMainView).suffix().toLower())) {
-
-        if(archiveContentPreloaded.length() > 0) {
-            m_entriesMainView = archiveContentPreloaded;
-            archiveContentPreloaded.clear();
+        int numberPageDocument = 0;
+        if(m_fileInFolderMainView.contains("::PDF::")) {
+            m_readDocumentOnly = true;
+            numberPageDocument = m_fileInFolderMainView.split("::PDF::").at(0).toInt();
+            m_fileInFolderMainView = m_fileInFolderMainView.split("::PDF::").at(1);
+        } else if(m_fileInFolderMainView.contains("::ARC::")) {
+            m_readArchiveOnly = true;
+            numberPageDocument = m_fileInFolderMainView.split("::ARC::").at(0).toInt();
+            m_fileInFolderMainView = m_fileInFolderMainView.split("::ARC::").at(1);
         }
-        m_countMainView = m_entriesMainView.length();
-        m_readArchiveOnly = false;
-        setCurrentIndex(numberPageDocument);
 
-    } else {
+        const bool isFolder = (QFile::exists(m_fileInFolderMainView) && !QFileInfo(m_fileInFolderMainView).isFile()) || m_fileInFolderMainView.endsWith("/");
 
-        m_entriesMainView = getAllFiles(isFolder ? m_fileInFolderMainView : QFileInfo(m_fileInFolderMainView).absolutePath());
-        m_countMainView = m_entriesMainView.length();
+        // clear any old watchers
+        if(watcherMainView->directories().length())
+            watcherMainView->removePaths(watcherMainView->directories());
+        if(watcherMainView->files().length())
+            watcherMainView->removePaths(watcherMainView->files());
 
-        if(isFolder && m_entriesMainView.length())
-            m_fileInFolderMainView = m_entriesMainView[0];
+        // and set the current one
+        watcherMainView->addPath(isFolder ? m_fileInFolderMainView : QFileInfo(m_fileInFolderMainView).absolutePath());
+        connect(watcherMainView, &QFileSystemWatcher::directoryChanged, this, [=]() { m_fileInFolderMainView = m_currentFile; loadDelayMainView->start(); });
+
+        if(m_readDocumentOnly) {// && PQCImageFormats::get().getEnabledFormatsPoppler().contains(QFileInfo(m_fileInFolderMainView).suffix().toLower())) {
+
+            m_entriesMainView = listPDFPages(m_fileInFolderMainView);
+            PQCFileFolderModelCPP::get().setEntriesMainView(m_entriesMainView);
+            setCountMainView(m_entriesMainView.length());
+            m_readDocumentOnly = false;
+            setCurrentIndex(numberPageDocument);
+
+            qDebug() << "Found" << m_entriesMainView.length() << "pages in document";
+
+        } else if(m_readArchiveOnly) {// && PQCImageFormats::get().getEnabledFormatsLibArchive().contains(QFileInfo(m_fileInFolderMainView).suffix().toLower())) {
+
+            if(archiveContentPreloaded.length() > 0) {
+                m_entriesMainView = archiveContentPreloaded;
+                PQCFileFolderModelCPP::get().setEntriesMainView(m_entriesMainView);
+                archiveContentPreloaded.clear();
+            }
+            setCountMainView(m_entriesMainView.length());
+            m_readArchiveOnly = false;
+            setCurrentIndex(numberPageDocument);
+
+            qDebug() << "Found" << m_entriesMainView.length() << "files in archive";
+
+        } else {
+
+            m_entriesMainView = getAllFiles(isFolder ? m_fileInFolderMainView : QFileInfo(m_fileInFolderMainView).absolutePath());
+            PQCFileFolderModelCPP::get().setEntriesMainView(m_entriesMainView);
+            setCountMainView(m_entriesMainView.length());
+
+            qDebug() << "Found" << m_entriesMainView.length() << "files";
+
+            if(isFolder && m_entriesMainView.length())
+                m_fileInFolderMainView = m_entriesMainView[0];
+
+        }
 
     }
 
     QFileInfo info(m_fileInFolderMainView);
     if(info.absolutePath() == cacheAdvancedSortFolderName)
 
-        advancedSortMainView();
+        advancedSortMainView(PQCSettingsCPP::get().getImageviewAdvancedSortCriteria(),
+                             PQCSettingsCPP::get().getImageviewAdvancedSortAscending(),
+                             PQCSettingsCPP::get().getImageviewAdvancedSortQuality(),
+                             PQCSettingsCPP::get().getImageviewAdvancedSortDateCriteria());
 
     else {
 
@@ -981,19 +1117,18 @@ void PQCFileFolderModel::loadDataMainView() {
 
         // we first make sure the count is set to 0
         // to force a refresh of the folder
-        const int tmp = m_countMainView;
-        m_countMainView = 0;
-        Q_EMIT countMainViewChanged();
-        m_countMainView = tmp;
+        const int tmp = getCountMainView();
+        setCountMainView(0);
+        setCountMainView(tmp);
 
         m_currentIndex = m_entriesMainView.indexOf(m_fileInFolderMainView);
         m_currentFile = m_fileInFolderMainView;
+        PQCFileFolderModelCPP::get().setCurrentFile(m_currentFile);
 
         m_currentIndexNoDelay = m_currentIndex;
         m_currentFileNoDelay = m_currentFile;
 
         Q_EMIT newDataLoadedMainView();
-        Q_EMIT countMainViewChanged();
         Q_EMIT currentIndexChanged();
         Q_EMIT currentFileChanged();
         Q_EMIT currentIndexNoDelayChanged();
@@ -1001,6 +1136,9 @@ void PQCFileFolderModel::loadDataMainView() {
 
     }
 
+    if(!m_firstFolderMainViewLoaded)
+        setFirstFolderMainViewLoaded(true);
+
 }
 
 void PQCFileFolderModel::loadDataFileDialog() {
@@ -1008,47 +1146,77 @@ void PQCFileFolderModel::loadDataFileDialog() {
     qDebug() << "";
 
     ////////////////////////
-    // clear old entries
+    // virtual folder
 
-    m_entriesFileDialog.clear();
-    m_countFoldersFileDialog = 0;
-    m_countFilesFileDialog = 0;
-    m_countAllFileDialog = 0;
-    delete watcherFileDialog;
-    watcherFileDialog = new QFileSystemWatcher;
+    bool virtualFolderLoaded = false;
+    bool virtualFolderEmpty = false;
+    if(m_loadVirtualFolderFileDialog) {
+
+        qDebug() << "Loading virtual folder:" << m_virtualFolders << "/" << m_virtualFiles;
+
+        QStringList allFiles = m_virtualFiles;
+        for(const QString &f : std::as_const(m_virtualFolders)) {
+
+            allFiles.append(getAllFiles(f));
+
+        }
+
+        m_entriesFileDialog = allFiles;
+        m_countFoldersFileDialog = 0;
+        m_countFilesFileDialog = allFiles.length();
+
+        if(allFiles.length() == 0)
+            virtualFolderEmpty = true;
+        else
+            virtualFolderLoaded = true;
+
+    }
 
     ////////////////////////
     // no new directory
 
-    if(m_folderFileDialog.isEmpty()) {
+    if(virtualFolderEmpty || m_folderFileDialog.isEmpty()) {
+
+        // clear old entries
+        m_entriesFileDialog.clear();
+        m_countFoldersFileDialog = 0;
+        m_countFilesFileDialog = 0;
+        m_countAllFileDialog = 0;
+
         Q_EMIT newDataLoadedFileDialog();
         Q_EMIT countFoldersFileDialogChanged();
         Q_EMIT countFilesFileDialogChanged();
         Q_EMIT countAllFileDialogChanged();
+
         return;
     }
 
-    ////////////////////////
-    // watch directory for changes
+    if(!virtualFolderLoaded) {
 
-    watcherFileDialog->addPath(m_folderFileDialog);
-    connect(watcherFileDialog, &QFileSystemWatcher::directoryChanged, this, &PQCFileFolderModel::loadDataFileDialog);
+        ////////////////////////
+        // watch directory for changes
 
-    ////////////////////////
-    // load folders
+        watcherFileDialog->addPath(m_folderFileDialog);
+        connect(watcherFileDialog, &QFileSystemWatcher::directoryChanged, this, &PQCFileFolderModel::loadDataFileDialog);
 
-    m_entriesFileDialog = getAllFolders(m_folderFileDialog);
-    m_countFoldersFileDialog = m_entriesFileDialog.length();
+        ////////////////////////
+        // load folders
 
-    ////////////////////////
-    // load files
+        m_entriesFileDialog = getAllFolders(m_folderFileDialog);
+        m_countFoldersFileDialog = m_entriesFileDialog.length();
 
-    m_entriesFileDialog.append(getAllFiles(m_folderFileDialog, true, true));
+        ////////////////////////
+        // load files
 
-    m_countFilesFileDialog = m_entriesFileDialog.length()-m_countFoldersFileDialog;
+        m_entriesFileDialog.append(getAllFiles(m_folderFileDialog, true, true));
+        m_countFilesFileDialog = m_entriesFileDialog.length()-m_countFoldersFileDialog;
+
+    }
 
     m_countAllFileDialog = m_countFoldersFileDialog+m_countFilesFileDialog;
 
+    qDebug() << "Found" << m_countFoldersFileDialog << "folders and" << m_countFilesFileDialog << "files";
+
     Q_EMIT newDataLoadedFileDialog();
     Q_EMIT countFoldersFileDialogChanged();
     Q_EMIT countFilesFileDialogChanged();
@@ -1062,9 +1230,9 @@ QStringList PQCFileFolderModel::getAllFolders(QString folder, bool forceShowHidd
 
     QStringList ret;
 
-    const bool sortReversed = !PQCSettings::get()["imageviewSortImagesAscending"].toBool();
-    QString sortBy = PQCSettings::get()["imageviewSortImagesBy"].toString();
-    const bool showHidden = (PQCSettings::get()["filedialogShowHiddenFilesFolders"].toBool() || forceShowHidden);
+    const bool sortReversed = !PQCSettingsCPP::get().getImageviewSortImagesAscending();
+    QString sortBy = PQCSettingsCPP::get().getImageviewSortImagesBy();
+    const bool showHidden = (PQCSettingsCPP::get().getFiledialogShowHiddenFilesFolders() || forceShowHidden);
 
 #ifdef PQMWITHOUTICU
     if(sortBy == "naturalname")
@@ -1128,9 +1296,9 @@ QStringList PQCFileFolderModel::getAllFiles(QString folder, bool ignoreFiltersEx
 
     QStringList ret;
 
-    const bool sortReversed = !PQCSettings::get()["imageviewSortImagesAscending"].toBool();
-    QString sortBy = PQCSettings::get()["imageviewSortImagesBy"].toString();
-    const bool showHidden = PQCSettings::get()["filedialogShowHiddenFilesFolders"].toBool();
+    const bool sortReversed = !PQCSettingsCPP::get().getImageviewSortImagesAscending();
+    QString sortBy = PQCSettingsCPP::get().getImageviewSortImagesBy();
+    const bool showHidden = PQCSettingsCPP::get().getFiledialogShowHiddenFilesFolders();
 
 #ifdef PQMWITHOUTICU
     if(sortBy == "naturalname")
@@ -1169,8 +1337,6 @@ QStringList PQCFileFolderModel::getAllFiles(QString folder, bool ignoreFiltersEx
                 if(count > 100)
                     break;
             }
-        } else if(m_extraFoldersToLoad.length() > 0) {
-            foldersToScan.append(m_extraFoldersToLoad);
         }
 
     }
@@ -1198,7 +1364,7 @@ QStringList PQCFileFolderModel::getAllFiles(QString folder, bool ignoreFiltersEx
 
             if(m_nameFilters.size() == 0 && m_restrictToSuffixes.size() == 0 && m_restrictToMimeTypes.size() == 0 && m_imageResolutionFilter.isNull() && m_fileSizeFilter == 0) {
                 const QFileInfoList lst = dir.entryInfoList();
-                if(PQCSettings::get()["filetypesLoadAppleLivePhotos"].toBool()) {
+                if(PQCSettingsCPP::get().getFiletypesLoadAppleLivePhotos()) {
                     for(const auto &f: lst) {
                         QFileInfo info(f);
                         // we need to exclude video files connected to Apple Live Videos (if support enabled)
@@ -1247,19 +1413,22 @@ QStringList PQCFileFolderModel::getAllFiles(QString folder, bool ignoreFiltersEx
 
                     }
 
-                    QString suffix = f.suffix().toLower();
-                    if(f.isSymLink() && f.exists())
-                        suffix = QFileInfo(f.symLinkTarget()).suffix().toLower();
-
-                    if((m_nameFilters.size() == 0 || (!ignoreFiltersExceptDefault && m_nameFilters.contains(suffix))) && (m_restrictToSuffixes.size() == 0 || m_restrictToSuffixes.contains(suffix))) {
+                    QString suffix1 = f.suffix().toLower();
+                    QString suffix2 = f.completeSuffix().toLower();
+                    if(f.isSymLink() && f.exists()) {
+                        suffix1 = QFileInfo(f.symLinkTarget()).suffix().toLower();
+                        suffix2 = QFileInfo(f.symLinkTarget()).completeSuffix().toLower();
+                    }
+                    if((m_nameFilters.size() == 0 || (!ignoreFiltersExceptDefault && (m_nameFilters.contains(suffix1) || m_nameFilters.contains(suffix2)))) &&
+                        (m_restrictToSuffixes.size() == 0 || m_restrictToSuffixes.contains(suffix1) || m_restrictToSuffixes.contains(suffix2))) {
                         if(m_filenameFilters.length() == 0 || ignoreFiltersExceptDefault) {
                             // we need to exclude video files connected to Apple Live Videos (if support enabled)
-                            if(PQCSettings::get()["filetypesLoadAppleLivePhotos"].toBool() && suffix == "mov" && QFileInfo::exists(f.absolutePath()+"/"+f.baseName()+".heic"))
+                            if(PQCSettingsCPP::get().getFiletypesLoadAppleLivePhotos() && suffix1 == "mov" && QFileInfo::exists(f.absolutePath()+"/"+f.baseName()+".heic"))
                                 continue;
                             ret_cur << f.absoluteFilePath();
                         } else {
                             // we need to exclude video files connected to Apple Live Videos (if support enabled)
-                            if(PQCSettings::get()["filetypesLoadAppleLivePhotos"].toBool() && suffix == "mov" && QFileInfo::exists(f.absolutePath()+"/"+f.baseName()+".heic"))
+                            if(PQCSettingsCPP::get().getFiletypesLoadAppleLivePhotos() && suffix1 == "mov" && QFileInfo::exists(f.absolutePath()+"/"+f.baseName()+".heic"))
                                 continue;
                             for(const QString &fil : std::as_const(m_filenameFilters)) {
                                 if(f.baseName().contains(fil)) {
@@ -1390,20 +1559,29 @@ void PQCFileFolderModel::enableViewerMode(int page) {
 
     qDebug() << "args: page =" << page;
 
+    m_activeViewerMode = true;
+    Q_EMIT activeViewerModeChanged();
+
     if(PQCScriptsImages::get().isPDFDocument(getCurrentFile()))
         setFileInFolderMainView(QString("%1::PDF::%2").arg(page).arg(getCurrentFile()));
     else {
-        archiveContentPreloaded = PQCScriptsImages::get().listArchiveContent(getCurrentFile());
+        archiveContentPreloaded = PQCScriptsImages::get().listArchiveContentWithoutThread(getCurrentFile());
         setFileInFolderMainView(QString("%1::ARC::%2").arg(archiveContentPreloaded[page], getCurrentFile()));
     }
     forceReloadMainView();
     setCurrentIndex(page);
 }
 
-void PQCFileFolderModel::disableViewerMode() {
+void PQCFileFolderModel::disableViewerMode(bool bufferDisabling) {
 
     qDebug() << "";
 
+    m_activeViewerMode = false;
+    Q_EMIT activeViewerModeChanged();
+
+    if(bufferDisabling)
+        m_justLeftViewerMode = true;
+
     QString tmp = getCurrentFile();
     if(tmp.contains("::PDF::"))
         setFileInFolderMainView(tmp.split("::PDF::")[1]);
@@ -1416,6 +1594,9 @@ void PQCFileFolderModel::disableViewerMode() {
     Q_EMIT isARCChanged();
     Q_EMIT isPDFChanged();
 
+    if(bufferDisabling)
+        timerResetJustLeftViewerMode->start();
+
 }
 
 QString PQCFileFolderModel::getFirstMatchFileDialog(QString partial) {
@@ -1445,3 +1626,37 @@ QString PQCFileFolderModel::getFirstMatchFileDialog(QString partial) {
     return "";
 
 }
+
+void PQCFileFolderModel::loadNextMatchOfSearch(const QString search) {
+
+    qDebug() << "args: search =" << search;
+
+    if(search == "") {
+        PQCNotifyCPP::get().showNotificationMessage("Nothing to search", "No search term to search for yet.");
+        return;
+    }
+
+    // first look forward
+    for(int i = m_currentIndex+1; i < m_countMainView; ++i) {
+        const QString cur = m_entriesMainView[i];
+        QFileInfo info(cur);
+        if(info.fileName().toLower().contains(search.toLower())) {
+            setFileInFolderMainView(cur);
+            return;
+        }
+    }
+
+    // then wrap around
+    for(int i = 0; i < m_currentIndex+1; ++i) {
+        const QString cur = m_entriesMainView[i];
+        QFileInfo info(cur);
+        if(info.fileName().toLower().contains(search.toLower())) {
+            setFileInFolderMainView(cur);
+            PQCNotifyCPP::get().showNotificationMessage("Wrapped around", "Search wrapped around from the beginning.");
+            return;
+        }
+    }
+
+    PQCNotifyCPP::get().showNotificationMessage("Nothing found", "The current search term returned no results.");
+
+}
