@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -23,15 +23,18 @@
 #include <qlogging.h>   // needed in this form to compile with Qt 6.2
 #include <QtDebug>
 #include <QFileDialog>
+#include <QApplication>
 #include <QImageReader>
 #include <QQmlContext>
 #include <QMessageBox>
+#include <QDirIterator>
 #include <pqc_configfiles.h>
-#include <pqc_settings.h>
-#include <pqc_shortcuts.h>
-#include <pqc_validate.h>
-#include <pqc_imageformats.h>
 #include <scripts/pqc_scriptsconfig.h>
+#include <scripts/pqc_scriptslocalization.h>
+#ifndef PQMTESTING
+#include <pqc_startuphandler.h>
+#endif
+#include <pqc_notify_cpp.h>
 
 #ifdef WIN32
 #include <WinSock2.h>
@@ -78,14 +81,9 @@
 #include <lcms2.h>
 #endif
 
-PQCScriptsConfig::PQCScriptsConfig() {
-    trans = new QTranslator;
-    currentTranslation = "en";
-}
+PQCScriptsConfig::PQCScriptsConfig() {}
 
-PQCScriptsConfig::~PQCScriptsConfig() {
-    delete trans;
-}
+PQCScriptsConfig::~PQCScriptsConfig() {}
 
 bool PQCScriptsConfig::amIOnWindows() {
 #ifdef Q_OS_WIN
@@ -113,6 +111,10 @@ QString PQCScriptsConfig::getConfigInfo(bool formatHTML) {
 
     txt += QString(" - Compiled with %1Qt %2%3, running with %4Qt %5%6%7").arg(bold1, QT_VERSION_STR, bold2, bold1, qVersion(), bold2, nl);
 
+#ifdef PQMPORTABLETWEAKS
+    txt += QString(" - Portable version%1").arg(nl);
+#endif
+
 #ifdef PQMEXIV2
     txt += QString(" - %1Exiv2%2: %3%4").arg(bold1, bold2, Exiv2::version(), nl);
 #endif
@@ -157,6 +159,10 @@ QString PQCScriptsConfig::getConfigInfo(bool formatHTML) {
     txt += QString(" - %1DevIL%2: %3%4").arg(bold1, bold2).arg(IL_VERSION).arg(nl);
 #endif
 
+#ifdef PQMLIBSAI
+    txt += QString(" - %1LibSai%2%3").arg(bold1, bold2).arg(nl);
+#endif
+
 #ifdef PQMLOCATION
     txt += QString(" - %1Location%2%3").arg(bold1, bold2, nl);
 #endif
@@ -197,6 +203,15 @@ QString PQCScriptsConfig::getConfigInfo(bool formatHTML) {
 
 bool PQCScriptsConfig::exportConfigTo(QString path) {
 
+    /*********************************************************/
+    // NOTE: BEFORE CALLING you HAVE TO ENSURE:
+    // -> settings database is closed
+    // -> shortcuts database is closed
+    //
+    // If called by command line, they are never opened.
+    // If called through interface, they must be closed (and subsequently reopened) manually!
+    /*********************************************************/
+
     qDebug() << "args: path =" << path;
 
 #ifdef PQMLIBARCHIVE
@@ -218,7 +233,7 @@ bool PQCScriptsConfig::exportConfigTo(QString path) {
 
     // All the config files
     QHash<QString,QString> allfiles;
-    allfiles["CFG_SETTINGS_DB"] = PQCConfigFiles::get().SETTINGS_DB();
+    allfiles["CFG_USERSETTINGS_DB"] = PQCConfigFiles::get().USERSETTINGS_DB();
     allfiles["CFG_IMAGEFORMATS_DB"] = PQCConfigFiles::get().IMAGEFORMATS_DB();
     allfiles["CFG_CONTEXTMENU_DB"] = PQCConfigFiles::get().CONTEXTMENU_DB();
     allfiles["CFG_SHORTCUTS_DB"] = PQCConfigFiles::get().SHORTCUTS_DB();
@@ -266,7 +281,7 @@ bool PQCScriptsConfig::exportConfigTo(QString path) {
                 archive_entry_free(entry);
 
             } else
-                qWarning() <<  QString("PQHandlingExternal::exportConfig(): ERROR: Unable to read config file '%1'... Skipping!").arg(iter.value());
+                qWarning() <<  QString("ERROR: Unable to read config file '%1'... Skipping!").arg(iter.value());
         }
         ++iter;
     }
@@ -284,14 +299,20 @@ bool PQCScriptsConfig::exportConfigTo(QString path) {
 
 bool PQCScriptsConfig::importConfigFrom(QString path) {
 
+    /*********************************************************/
+    // NOTE: BEFORE CALLING you HAVE TO ENSURE:
+    // -> settings database is closed
+    // -> shortcuts database is closed
+    // -> image formats database is closed
+    //
+    // If called by command line, they are never opened.
+    // If called through interface, they must be closed manually!
+    /*********************************************************/
+
     qDebug() << "args: path =" << path;
 
 #ifdef PQMLIBARCHIVE
 
-    PQCSettings::get().closeDatabase();
-    PQCShortcuts::get().closeDatabase();
-    PQCImageFormats::get().closeDatabase();
-
     // Obtain a filename from the user or used passed on filename
     QString archiveFile;
     if(path == "") {
@@ -306,7 +327,9 @@ bool PQCScriptsConfig::importConfigFrom(QString path) {
 
     // All the config files to be imported
     QHash<QString,QString> allfiles;
-    allfiles["CFG_SETTINGS_DB"] = PQCConfigFiles::get().SETTINGS_DB();
+    // the old settings file CAN be used by the new versions (but not the other way round)
+    allfiles["CFG_SETTINGS_DB"] = PQCConfigFiles::get().USERSETTINGS_DB();
+    allfiles["CFG_USERSETTINGS_DB"] = PQCConfigFiles::get().USERSETTINGS_DB();
     allfiles["CFG_CONTEXTMENU_DB"] = PQCConfigFiles::get().CONTEXTMENU_DB();
     allfiles["CFG_SHORTCUTS_DB"] = PQCConfigFiles::get().SHORTCUTS_DB();
     allfiles["CFG_IMAGEFORMATS_DB"] = PQCConfigFiles::get().IMAGEFORMATS_DB();
@@ -529,108 +552,27 @@ QString PQCScriptsConfig::getVersion() {
     return PQMVERSION;
 }
 
-QStringList PQCScriptsConfig::getAvailableTranslations() {
-
-    qDebug() << "";
-
-    QStringList ret;
-
-    QStringList tmp;
-
-    // the non-translated language is English
-    tmp << "en";
-
-    QDirIterator it(":/lang");
-    while (it.hasNext()) {
-        QString file = it.next();
-        if(file.endsWith(".qm")) {
-            file = file.remove(0, 15);
-            file = file.remove(file.length()-3, file.length());
-            if(!ret.contains(file))
-                tmp.push_back(file);
-        }
-    }
-
-    tmp.sort();
-    ret.append(tmp);
-
-    return ret;
-
-}
-
-void PQCScriptsConfig::updateTranslation() {
-
-    qDebug() << "";
-
-    QString code = PQCSettings::get()["interfaceLanguage"].toString();
-    if(code == currentTranslation)
-        return;
-
-    static QTranslator trans;
-    qApp->removeTranslator(&trans);
-
-    const QStringList allcodes = code.split("/");
-
-    // we use this to detect whether a translation was found for the above language code
-    currentTranslation = "";
-    for(const QString &c : allcodes) {
-
-        if(QFile(":/lang/photoqt_" + c + ".qm").exists()) {
-
-            if(trans.load(":/lang/photoqt_" + c)) {
-                currentTranslation = c;
-                qApp->installTranslator(&trans);
-            } else
-                qWarning() << "Unable to install translator for language code" << c;
-
-        } else if(c.contains("_")) {
-
-            const QString cc = c.split("_").at(0);
-
-            if(QFile(":/lang/photoqt_" + cc + ".qm").exists()) {
-
-                if(trans.load(":/lang/photoqt_" + cc)) {
-                    currentTranslation = cc;
-                    qApp->installTranslator(&trans);
-                } else
-                    qWarning() << "Unable to install translator for language code" << cc;
-
-            }
-
-        } else {
-
-            const QString cc = QString("%1_%2").arg(c, c.toUpper());
-
-            if(QFile(":/lang/photoqt_" + cc + ".qm").exists()) {
-
-                if(trans.load(":/lang/photoqt_" + cc)) {
-                    currentTranslation = cc;
-                    qApp->installTranslator(&trans);
-                } else
-                    qWarning() << "Unable to install translator for language code" << c;
-
-            }
-        }
-
-    }
-
-    // no translation found -> store selected code
-    if(currentTranslation == "")
-        currentTranslation = code;
-
-    QQmlEngine::contextForObject(this)->engine()->retranslate();
-
-}
-
 bool PQCScriptsConfig::isBetaVersion() {
     return QString(PQMVERSION).contains("-beta");
 }
 
+bool PQCScriptsConfig::isDebugBuild() {
+#ifdef NDEBUG
+    return false;
+#else
+    return true;
+#endif
+}
+
 void PQCScriptsConfig::inform(QString title, QString txt) {
 
-    QMessageBox::information(0,
-                             title,
-                             txt);
+    QMessageBox msg;
+    msg.setIcon(QMessageBox::Information);
+    msg.setWindowFlag(Qt::WindowStaysOnTopHint);
+    msg.setWindowTitle(title);
+    msg.setText(txt);
+    msg.setStandardButtons(QMessageBox::Ok);
+    msg.exec();
 
 }
 
@@ -662,13 +604,56 @@ bool PQCScriptsConfig::isLCMS2SupportEnabled() {
     return false;
 }
 
-void PQCScriptsConfig::setDefaultSettingValueFor(QString key) {
-    PQCSettings::get().setDefaultFor(key);
-}
-
 bool PQCScriptsConfig::isICUSupportEnabled() {
 #ifdef PQMWITHOUTICU
     return false;
 #endif
     return true;
 }
+
+void PQCScriptsConfig::callStartupSetupFresh() {
+
+#ifndef PQMTESTING
+    PQCStartupHandler startup(false, false);
+    startup.setupFresh();
+#endif
+
+}
+
+bool PQCScriptsConfig::askForConfirmation(QString title, QString text, QString informativeText) {
+    QMessageBox msg;
+    msg.setIcon(QMessageBox::Question);
+    msg.setWindowFlag(Qt::WindowStaysOnTopHint);
+    msg.setWindowTitle(title);
+    msg.setText(text);
+    msg.setInformativeText(informativeText);
+    msg.setStandardButtons(QMessageBox::Yes | QMessageBox::Cancel);
+    msg.setDefaultButton(QMessageBox::Yes);
+    return (msg.exec() == QMessageBox::Yes);
+}
+
+bool PQCScriptsConfig::setInterfaceForNextStartup(QString variant) {
+    QFile file(PQCConfigFiles::get().CACHE_DIR() + "/nextstartupvariant");
+    if(!file.open(QIODevice::WriteOnly|QIODevice::Truncate)) {
+        qWarning() << "Unable to toggle interface variant";
+        return false;
+    }
+    QTextStream out(&file);
+    out << variant;
+    file.close();
+    return true;
+}
+
+QString PQCScriptsConfig::getInterfaceForNextStartup() {
+    QFile file(PQCConfigFiles::get().CACHE_DIR() + "/nextstartupvariant");
+    if(!file.exists()) return "";
+    if(!file.open(QIODevice::ReadOnly)) {
+        qWarning() << "Unable to read toggled interface variant";
+        return "";
+    }
+    QTextStream in(&file);
+    QString ret = in.readAll().trimmed();
+    if(ret == "modern" || ret == "integrated")
+        return ret;
+    return "";
+}
