@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -35,15 +35,20 @@
 PQCScriptsContextMenu::PQCScriptsContextMenu() {
 
     // connect to database
-    if(QSqlDatabase::isDriverAvailable("QSQLITE3"))
-        db = QSqlDatabase::addDatabase("QSQLITE3", "contextmenu");
-    else if(QSqlDatabase::isDriverAvailable("QSQLITE"))
-        db = QSqlDatabase::addDatabase("QSQLITE", "contextmenu");
-    db.setDatabaseName(PQCConfigFiles::get().CONTEXTMENU_DB());
+    db = QSqlDatabase::database("contextmenu");
 
     QFileInfo infodb(PQCConfigFiles::get().CONTEXTMENU_DB());
 
-    if(!infodb.exists() || !db.open()) {
+    if(!infodb.exists()) {
+        if(!QFile::copy(":/contextmenu.db", PQCConfigFiles::get().CONTEXTMENU_DB()))
+            qWarning() << "Unable to (re-)create default contextmenu database";
+        else {
+            QFile file(PQCConfigFiles::get().CONTEXTMENU_DB());
+            file.setPermissions(file.permissions()|QFileDevice::WriteOwner);
+        }
+    }
+
+    if(!db.open()) {
 
         qWarning() << "ERROR opening database:" << db.lastError().text();
         qWarning() << "no context menu entries will be available";
@@ -204,3 +209,11 @@ QVariantList PQCScriptsContextMenu::detectSystemEntries() {
     return ret;
 
 }
+
+void PQCScriptsContextMenu::closeDatabase() {
+
+    qDebug() << "";
+
+    db.close();
+
+}
