@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -21,8 +21,8 @@
  **************************************************************************/
 
 #include <scripts/pqc_scriptsfilespaths.h>
-#include <pqc_settings.h>
-#include <pqc_notify.h>
+#include <pqc_settingscpp.h>
+#include <pqc_notify_cpp.h>
 #include <pqc_imageformats.h>
 #include <pqc_configfiles.h>
 #include <qlogging.h>   // needed in this form to compile with Qt 6.2
@@ -34,13 +34,50 @@
 #include <QCollator>
 #include <QDesktopServices>
 #include <QFileDialog>
+#include <QProcess>
+
+PQCScriptsFilesPaths &PQCScriptsFilesPaths::get() {
+    static PQCScriptsFilesPaths instance;
+    return instance;
+}
 
 PQCScriptsFilesPaths::PQCScriptsFilesPaths() {
     animatedImageTemporaryCounter = 0;
+
+    networkSharesTimer.setInterval(1000*60*5);
+    connect(&networkSharesTimer, &QTimer::timeout, this, &PQCScriptsFilesPaths::detectNetworkShares);
+    detectNetworkShares();
 }
 
-PQCScriptsFilesPaths::~PQCScriptsFilesPaths() {
+PQCScriptsFilesPaths::~PQCScriptsFilesPaths() { }
 
+void PQCScriptsFilesPaths::detectNetworkShares() {
+    networkshares.clear();
+    const QList<QStorageInfo> info = QStorageInfo::mountedVolumes();
+    for(const QStorageInfo &s : info) {
+        if(s.isValid() && (s.fileSystemType() == "cifs" || s.fileSystemType() == "samba" || s.fileSystemType() == "fuse"))
+            networkshares.push_back(s.rootPath());
+#ifdef Q_OS_WIN
+        // on windows network shares often have a fileSystemType of FAT or NTFS or therelike
+        // This check excludes known physical devices assuming everything else to be remote
+        if (!QString::fromLatin1(s.device()).startsWith(QLatin1String("\\\\?\\Volume")))
+            networkshares.push_back(s.rootPath());
+#endif
+    }
+#ifdef Q_OS_UNIX
+    // sshfs mounts are not listed as part of mountedVolumes but we might be able to find them in mtab
+    QFile f("/etc/mtab");
+    if(f.exists() && f.open(QIODevice::ReadOnly)) {
+        QTextStream in(&f);
+        QString line;
+        while(in.readLineInto(&line)) {
+            QStringList parts = line.split(" ");
+            if(parts[2] == "fuse.sshfs")
+                networkshares.push_back(parts[1]);
+        }
+    }
+#endif
+    networkSharesTimer.start();
 }
 
 QString PQCScriptsFilesPaths::cleanPath(QString path) {
@@ -129,10 +166,28 @@ QString PQCScriptsFilesPaths::pathFromNativeSeparators(QString path) {
 
 QString PQCScriptsFilesPaths::getSuffix(QString path) {
 
+    return QFileInfo(path).suffix();
+
+}
+
+QString PQCScriptsFilesPaths::getSuffixLowerCase(QString path) {
+
+    return QFileInfo(path).suffix().toLower();
+
+}
+
+QString PQCScriptsFilesPaths::getCompleteSuffix(QString path) {
+
     return QFileInfo(path).completeSuffix();
 
 }
 
+QString PQCScriptsFilesPaths::getCompleteSuffixLowerCase(QString path) {
+
+    return QFileInfo(path).completeSuffix().toLower();
+
+}
+
 QString PQCScriptsFilesPaths::getBasename(QString fullpath) {
 
     if(fullpath == "")
@@ -159,10 +214,25 @@ QString PQCScriptsFilesPaths::getDir(QString fullpath) {
     if(fullpath == "")
         return "";
 
+    if(fullpath.contains("::ARC::"))
+        return QFileInfo(fullpath.split("::ARC::")[1]).absolutePath();
+    if(fullpath.contains("::PDF::"))
+        return QFileInfo(fullpath.split("::PDF::")[1]).absolutePath();
+
     return QFileInfo(fullpath).absolutePath();
 
 }
 
+QString PQCScriptsFilesPaths::getDirname(const QString fullpath) {
+
+    if(fullpath == "")
+        return "";
+
+    QDir dir(fullpath);
+    return dir.dirName();
+
+}
+
 QDateTime PQCScriptsFilesPaths::getFileModified(QString path) {
 
     return QFileInfo(path).lastModified();
@@ -195,10 +265,18 @@ QString PQCScriptsFilesPaths::getFileSizeHumanReadable(QString path) {
 
 }
 
+double PQCScriptsFilesPaths::convertBytesToGB(const qint64 bytes) {
+    return qRound(100.0*(bytes/(1024.0*1024.0*1024.0)))/100.0;
+}
+
 QString PQCScriptsFilesPaths::toPercentEncoding(QString str) {
     return QUrl::toPercentEncoding(str);
 }
 
+QString PQCScriptsFilesPaths::fromPercentEncoding(QString str) {
+    return QUrl::fromPercentEncoding(str.toUtf8());
+}
+
 QString PQCScriptsFilesPaths::goUpOneLevel(QString path) {
     QDir dir(path);
     dir.cdUp();
@@ -259,31 +337,48 @@ bool PQCScriptsFilesPaths::doesItExist(QString path) {
 
 bool PQCScriptsFilesPaths::isExcludeDirFromCaching(QString filename) {
 
-    if(PQCSettings::get()["thumbnailsExcludeDropBox"].toString() != "") {
-        if(filename.indexOf(PQCSettings::get()["thumbnailsExcludeDropBox"].toString())== 0)
+    qDebug() << "args: filename =" << filename;
+
+    if(PQCSettingsCPP::get().getThumbnailsExcludeDropBox() != "") {
+        if(filename.indexOf(PQCSettingsCPP::get().getThumbnailsExcludeDropBox())== 0)
             return true;
     }
 
-    if(PQCSettings::get()["thumbnailsExcludeNextcloud"].toString() != "") {
-        if(filename.indexOf(PQCSettings::get()["thumbnailsExcludeNextcloud"].toString())== 0)
+    if(PQCSettingsCPP::get().getThumbnailsExcludeNextcloud() != "") {
+        if(filename.indexOf(PQCSettingsCPP::get().getThumbnailsExcludeNextcloud())== 0)
             return true;
     }
 
-    if(PQCSettings::get()["thumbnailsExcludeOwnCloud"].toString() != "") {
-        if(filename.indexOf(PQCSettings::get()["thumbnailsExcludeOwnCloud"].toString())== 0)
+    if(PQCSettingsCPP::get().getThumbnailsExcludeOwnCloud() != "") {
+        if(filename.indexOf(PQCSettingsCPP::get().getThumbnailsExcludeOwnCloud())== 0)
             return true;
     }
 
-    const QStringList str = PQCSettings::get()["thumbnailsExcludeFolders"].toStringList();
+    const QStringList str = PQCSettingsCPP::get().getThumbnailsExcludeFolders();
     for(const QString &dir: str) {
         if(dir != "" && filename.indexOf(dir) == 0)
             return true;
     }
 
+    if(PQCSettingsCPP::get().getThumbnailsExcludeNetworkShares()) {
+        return isOnNetwork(filename);
+    }
+
     return false;
 
 }
 
+bool PQCScriptsFilesPaths::isOnNetwork(QString filename) {
+
+    qDebug() << "args: filename =" << filename;
+
+    for(const QString &dir: std::as_const(networkshares)) {
+        if(dir != "" && filename.indexOf(dir) == 0)
+            return true;
+    }
+    return false;
+}
+
 void PQCScriptsFilesPaths::openInDefaultFileManager(QString filename) {
 
     qDebug() << "args: filename =" << filename;
@@ -350,6 +445,12 @@ void PQCScriptsFilesPaths::openInDefaultFileManager(QString filename) {
 
 }
 
+QString PQCScriptsFilesPaths::selectFileFromDialog(QString buttonlabel, QString preselectFile, bool confirmOverwrite) {
+
+    return PQCScriptsFilesPaths::selectFileFromDialog(buttonlabel, preselectFile, PQCImageFormats::get().detectFormatId(preselectFile), confirmOverwrite);
+
+}
+
 QString PQCScriptsFilesPaths::selectFileFromDialog(QString buttonlabel, QString preselectFile, int formatId, bool confirmOverwrite) {
 
     qDebug() << "args: buttonlabel" << buttonlabel;
@@ -359,11 +460,10 @@ QString PQCScriptsFilesPaths::selectFileFromDialog(QString buttonlabel, QString
 
     QFileInfo info(preselectFile);
 
-    PQCNotify::get().setModalFileDialogOpen(true);
-
     const QStringList endings = PQCImageFormats::get().getFormatEndings(formatId);
 
     QFileDialog diag;
+    diag.setWindowModality(Qt::ApplicationModal);
     diag.setLabelText(QFileDialog::Accept, buttonlabel);
     diag.setFileMode(QFileDialog::AnyFile);
     diag.setModal(true);
@@ -378,7 +478,6 @@ QString PQCScriptsFilesPaths::selectFileFromDialog(QString buttonlabel, QString
     if(diag.exec()) {
         QStringList fileNames = diag.selectedFiles();
         if(fileNames.length() > 0) {
-            PQCNotify::get().setModalFileDialogOpen(false);
             QString fn = fileNames[0];
             QFileInfo newinfo(fn);
             if(newinfo.suffix() == "")
@@ -387,7 +486,6 @@ QString PQCScriptsFilesPaths::selectFileFromDialog(QString buttonlabel, QString
         }
     }
 
-    PQCNotify::get().setModalFileDialogOpen(false);
     return "";
 
 }
@@ -399,7 +497,6 @@ QString PQCScriptsFilesPaths::selectFolderFromDialog(QString buttonlabel, QStrin
 
     QFileInfo info(preselectFolder);
 
-    PQCNotify::get().setModalFileDialogOpen(true);
 
     QFileDialog diag;
     diag.setLabelText(QFileDialog::Accept, buttonlabel);
@@ -412,12 +509,10 @@ QString PQCScriptsFilesPaths::selectFolderFromDialog(QString buttonlabel, QStrin
     if(diag.exec()) {
         QStringList fileNames = diag.selectedFiles();
         if(fileNames.length() > 0) {
-            PQCNotify::get().setModalFileDialogOpen(false);
             return fileNames[0];
         }
     }
 
-    PQCNotify::get().setModalFileDialogOpen(false);
     return "";
 
 }
@@ -426,23 +521,21 @@ void PQCScriptsFilesPaths::saveLogToFile(QString txt) {
 
     qDebug() << "args: txt.length = " << txt.length();
 
-    PQCNotify::get().setModalFileDialogOpen(true);
-
     QString newfile = QFileDialog::getSaveFileName(nullptr, QString(), QString("%1/photoqt-%2.log").arg(QDir::homePath(), QDateTime::currentDateTime().toString("yyyy-MM-dd_hh-mm")));
 
     if(newfile == "") {
-        PQCNotify::get().setModalFileDialogOpen(false);
         return;
     }
 
     QFile file(newfile);
-    file.open(QIODevice::WriteOnly|QIODevice::Truncate);
+    if(!file.open(QIODevice::WriteOnly|QIODevice::Truncate)) {
+        qWarning() << "ERROR opening file for saving log.";
+        return;
+    }
     QTextStream out(&file);
     out << txt;
     file.close();
 
-    PQCNotify::get().setModalFileDialogOpen(false);
-
 }
 
 QString PQCScriptsFilesPaths::openFileFromDialog(QString buttonlabel, QString preselectFile, QStringList endings) {
@@ -468,8 +561,6 @@ QStringList PQCScriptsFilesPaths::openFilesFromDialog(QString buttonlabel, QStri
 
     QFileInfo info(preselectFile);
 
-    PQCNotify::get().setModalFileDialogOpen(true);
-
     QFileDialog diag;
     diag.setLabelText(QFileDialog::Accept, buttonlabel);
     diag.setFileMode(QFileDialog::ExistingFiles);
@@ -487,15 +578,12 @@ QStringList PQCScriptsFilesPaths::openFilesFromDialog(QString buttonlabel, QStri
     if(diag.exec()) {
         QStringList fileNames = diag.selectedFiles();
         if(fileNames.length() > 0) {
-            PQCNotify::get().setModalFileDialogOpen(false);
             return fileNames;
         }
     }
 
-    PQCNotify::get().setModalFileDialogOpen(false);
     return QStringList();
 
-
 }
 
 QString PQCScriptsFilesPaths::createTooltipFilename(QString fname) {
@@ -583,8 +671,7 @@ QString PQCScriptsFilesPaths::findDropBoxFolder() {
 #else
     QFile f(QString("%1/Dropbox/host.db").arg(QStandardPaths::AppDataLocation));
 #endif
-    if(f.exists()) {
-        f.open(QIODevice::ReadOnly);
+    if(f.exists() && f.open(QIODevice::ReadOnly)) {
         QTextStream in(&f);
         QStringList txt = in.readAll().split("\n");
         if(txt.length() > 1) {
@@ -608,8 +695,7 @@ QString PQCScriptsFilesPaths::findNextcloudFolder() {
 #elif defined Q_OS_WIN
     QFile f(QString("%1/Nextcloud/nextcloud.cfg").arg(QStandardPaths::AppDataLocation));
 #endif
-    if(f.exists()) {
-        f.open(QIODevice::ReadOnly);
+    if(f.exists() && f.open(QIODevice::ReadOnly)) {
         QTextStream in(&f);
         QString txt = in.readAll();
         if(txt.contains("0\\Folders\\1\\localPath=")) {
@@ -633,8 +719,7 @@ QString PQCScriptsFilesPaths::findOwnCloudFolder() {
 #elif defined Q_OS_WIN
     QFile f(QString("%1/ownCloud/owncloud.cfg").arg(QStandardPaths::AppDataLocation));
 #endif
-    if(f.exists()) {
-        f.open(QIODevice::ReadOnly);
+    if(f.exists() && f.open(QIODevice::ReadOnly)) {
         QTextStream in(&f);
         QString txt = in.readAll();
         if(txt.contains("0\\Folders\\1\\localPath=")) {
@@ -730,3 +815,159 @@ void PQCScriptsFilesPaths::setThumbnailBaseCacheDir(QString dir) {
     PQCConfigFiles::get().setThumbnailCacheBaseDir(dir);
 
 }
+
+QString PQCScriptsFilesPaths::getSiblingFile(const QString currentFile, const int direction, int &remainingIteration, int &remainingLevelUp, int &remainingLevelDown) {
+
+    QString siblingFile = "";
+
+    QDir prefixDir(QFileInfo(currentFile).dir().absolutePath());
+
+    /////////////////////////////////////////////////////////////////////////
+    // Loop until we found a file or exceeded maximum counter
+
+    while(siblingFile == "" && remainingIteration > 0 && remainingLevelUp > 0) {
+
+        /////////////////////////////////////////////////////////////////////////
+        /// // Go up a folder to find siblings
+
+        // this is where we are currently in (stored so we don't check for files in ourselves)
+        QString origDirName = prefixDir.dirName();
+
+        // go up a level
+        if(!prefixDir.cdUp())
+            break;
+        remainingLevelDown += 1;
+        remainingLevelUp -= 1;
+
+        // we exclude root, this can be expensive
+        if(prefixDir.isRoot())
+            break;
+
+        qDebug() << "Going up to folder:" << prefixDir.absolutePath();
+
+        /////////////////////////////////////////////////////////////////////////
+        // Get all directories in parent directory (siblings to this folder)
+
+        QDir parentDir(prefixDir.absolutePath());
+        QStringList parentSiblings = parentDir.entryList(QDir::Dirs|QDir::NoDotAndDotDot);
+        _sortList(parentSiblings, true);
+
+        // our position in the directory
+        const int currentIndex = parentSiblings.indexOf(origDirName);
+
+        qDebug() << "currentIndex =" << currentIndex;
+
+        /////////////////////////////////////////////////////////////////////////
+        // check previous/next folders for content
+
+        // go backwards
+        if(direction == -1 && currentIndex > 0) {
+
+            // check all previous directories one by one
+            for(int i = currentIndex-1; i >= 0; --i) {
+                siblingFile = _findFirstFileinFolderAndSubFolder(prefixDir.absolutePath() + "/" + parentSiblings.at(i), false, remainingIteration, remainingLevelDown);
+                // if we found a file or reached maximum iteration level: stop
+                if(siblingFile != "" || remainingIteration <= 0) break;
+            }
+
+        // go forwards
+        } else if(direction == 1 && currentIndex < parentSiblings.length()-1) {
+
+            // check all next directories one by one
+            for(int i = currentIndex+1; i < parentSiblings.length(); ++i) {
+                siblingFile = _findFirstFileinFolderAndSubFolder(prefixDir.absolutePath() + "/" + parentSiblings.at(i), true, remainingIteration, remainingLevelDown);
+                // if we found a file or reached maximum iteration level: stop
+                if(siblingFile != "" || remainingIteration <= 0) break;
+            }
+
+        }
+
+        // if nothing found one level up, repeat
+
+    }
+
+    return siblingFile;
+
+}
+
+QString PQCScriptsFilesPaths::_findFirstFileinFolderAndSubFolder(const QString folder, const bool ascendingFolder, int &remainingIteration, int &remainingDescending) {
+
+    qDebug() << "Checking folder:" << folder;
+
+    remainingIteration -= 1;
+
+    QString ret = "";
+
+    QDir dir(folder);
+
+    QStringList fileList = dir.entryList(QDir::Files);
+
+    if(fileList.length() > 0) {
+
+        qDebug() << "Found" << fileList.length() << "files, checking for supported file types";
+
+        _sortList(fileList, true);
+
+        for(const QString &f : std::as_const(fileList)) {
+
+            const QString fullPath = dir.absolutePath() + "/" + f;
+
+            const QString suffix = QFileInfo(fullPath).suffix().toLower();
+            if(PQCImageFormats::get().getEnabledFormats().contains(suffix)) {
+                ret = fullPath;
+                break;
+            } else {
+                QMimeDatabase db;
+                QString mimetype = db.mimeTypeForFile(fullPath).name();
+                if(PQCImageFormats::get().getEnabledMimeTypes().contains(mimetype)) {
+                    ret = fullPath;
+                    break;
+                }
+            }
+
+        }
+
+    }
+
+    if(ret == "" && remainingDescending > 0) {
+
+        QStringList folderList = dir.entryList(QDir::Dirs|QDir::NoDotAndDotDot);
+
+        if(folderList.length() > 0) {
+
+            qDebug() << "Found" << folderList.length() << "subfolder, checking their contents";
+
+            _sortList(folderList, ascendingFolder);
+
+            for(const QString &f : std::as_const(folderList)) {
+
+                remainingDescending -= 1;
+                ret = _findFirstFileinFolderAndSubFolder(dir.absolutePath() + "/" + f, ascendingFolder, remainingIteration, remainingDescending);
+
+                if(ret != "")
+                    break;
+
+            }
+
+        }
+
+    }
+
+    return ret;
+
+}
+
+void PQCScriptsFilesPaths::_sortList(QStringList &lst, const bool ascending) {
+    QCollator collator;
+    collator.setNumericMode(true);
+    if(ascending)
+        std::sort(lst.begin(), lst.end(), [&collator](const QString &file1, const QString &file2) { return collator.compare(file1, file2) < 0; });
+    else
+        std::sort(lst.begin(), lst.end(), [&collator](const QString &file1, const QString &file2) { return collator.compare(file2, file1) < 0; });
+}
+
+bool PQCScriptsFilesPaths::areDirsTheSame(QString folder1, QString folder2) {
+    qDebug() << "args: folder1 =" << folder1;
+    qDebug() << "args: folder2 =" << folder2;
+    return (QDir(folder1)==QDir(folder2));
+}
