@@ -1,6 +1,6 @@
 /**************************************************************************
  **                                                                      **
- ** Copyright (C) 2011-2025 Lukas Spies                                  **
+ ** Copyright (C) 2011-2026 Lukas Spies                                  **
  ** Contact: https://photoqt.org                                         **
  **                                                                      **
  ** This file is part of PhotoQt.                                        **
@@ -23,8 +23,11 @@
 #include <scripts/pqc_scriptsmetadata.h>
 #include <pqc_configfiles.h>
 #include <pqc_imageformats.h>
+#include <pqc_settingscpp.h>
 
 #include <QtDebug>
+#include <QDateTime>
+#include <QLocale>
 #include <QPointF>
 #include <QVariant>
 #include <QFileInfo>
@@ -33,24 +36,42 @@
 #include <exiv2/exiv2.hpp>
 #endif
 
-PQCScriptsMetaData::PQCScriptsMetaData() {
-
+PQCScriptsMetaData &PQCScriptsMetaData::get() {
+    static PQCScriptsMetaData instance;
+    return instance;
 }
 
-PQCScriptsMetaData::~PQCScriptsMetaData() {
+PQCScriptsMetaData::PQCScriptsMetaData() {}
+PQCScriptsMetaData::~PQCScriptsMetaData() {}
 
-}
+QString PQCScriptsMetaData::analyzeDateTimeOriginal(const QString dateTime, const QString offset) {
 
-QString PQCScriptsMetaData::analyzeDateTimeOriginal(const QString val) {
+    qDebug() << "args: dateTime =" << dateTime;
+    qDebug() << "args: offset =" << offset;
 
-    qDebug() << "args: val =" << val;
+    QLocale locale = QLocale(PQCSettingsCPP::get().getInterfaceLanguage());
 
-    QStringList split1 = val.split(" ");
-    QStringList split2 = split1.at(0).split(":");
-    if(split1.length() > 1 && split2.length() > 2)
-        return split2.at(2) + "/" + split2.at(1) + "/" + split2.at(0) + ", " + split1.at(1);
+    QString inputFormat = "yyyy:MM:dd HH:mm:ss";
+    if(!offset.isEmpty())
+        inputFormat += "ttt";
 
-    return val;
+    QString combined = dateTime + offset;
+    QDateTime dt = locale.toDateTime(combined, inputFormat);
+
+    if(dt.isValid()) {
+
+        QString outputFormat = locale.dateTimeFormat();
+
+        // If the EXIF data doesn't contain the time zone offset, don't show the user a potentially
+        // invalid time zone.
+        if(offset.isEmpty())
+            outputFormat = outputFormat.replace("t", "").trimmed();
+
+        return locale.toString(dt, outputFormat);
+
+    }
+
+    return combined;
 
 }
 
@@ -487,7 +508,7 @@ QString PQCScriptsMetaData::convertGPSDecimalToDegree(double lat, double lon) {
     // find seconds
     double lat_sec = qRound((lat_min_full-lat_min)*60*1e3)/1e3;
     double lon_sec = qRound((lon_min_full-lon_min)*60*1e3)/1e3;
-    
+
     // assemble final string
     ret = QString("%1° %2' %3''%4, %5° %6' %7''%8").arg(lat_deg).arg(lat_min).arg(lat_sec).arg(lat_dir)
                                                    .arg(lon_deg).arg(lon_min).arg(lon_sec).arg(lon_dir);
